#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Jon Levell <levell@uk.ibm.com>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# All rights reserved.

# This shows a example of an MQTT subscriber with the ability to use
# user name, password CA certificates based on command line arguments

import paho.mqtt.client as mqtt
import os
import ssl
import argparse

parser = argparse.ArgumentParser()

parser.add_argument('-H', '--host', required=False, default="mqtt.eclipse.org")
parser.add_argument('-t', '--topic', required=False, default="$SYS/#")
parser.add_argument('-q', '--qos', required=False, type=int, default=0)
parser.add_argument('-c', '--clientid', required=False, default=None)
parser.add_argument('-u', '--username', required=False, default=None)
parser.add_argument('-d', '--disable-clean-session', action='store_true', help="disable 'clean session' (sub + msgs not cleared when client disconnects)")
parser.add_argument('-p', '--password', required=False, default=None)
parser.add_argument('-P', '--port', required=False, type=int, default=None, help='Defaults to 8883 for TLS or 1883 for non-TLS')
parser.add_argument('-k', '--keepalive', required=False, type=int, default=60)
parser.add_argument('-s', '--use-tls', action='store_true')
parser.add_argument('--insecure', action='store_true')
parser.add_argument('-F', '--cacerts', required=False, default=None)
parser.add_argument('--tls-version', required=False, default=None, help='TLS protocol version, can be one of tlsv1.2 tlsv1.1 or tlsv1\n')
parser.add_argument('-D', '--debug', action='store_true')

args, unknown = parser.parse_known_args()


def on_connect(mqttc, obj, flags, rc):
    print("rc: " + str(rc))


def on_message(mqttc, obj, msg):
    print(msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


def on_publish(mqttc, obj, mid):
    print("mid: " + str(mid))


def on_subscribe(mqttc, obj, mid, granted_qos):
    print("Subscribed: " + str(mid) + " " + str(granted_qos))


def on_log(mqttc, obj, level, string):
    print(string)

usetls = args.use_tls

if args.cacerts:
    usetls = True

port = args.port    
if port is None:
    if usetls:
        port = 8883
    else:
        port = 1883

mqttc = mqtt.Client(args.clientid,clean_session = not args.disable_clean_session)

if usetls:
    if args.tls_version == "tlsv1.2":
       tlsVersion = ssl.PROTOCOL_TLSv1_2
    elif args.tls_version == "tlsv1.1":
       tlsVersion = ssl.PROTOCOL_TLSv1_1
    elif args.tls_version == "tlsv1":
       tlsVersion = ssl.PROTOCOL_TLSv1
    elif args.tls_version is None:
       tlsVersion = None
    else:
       print ("Unknown TLS version - ignoring")
       tlsVersion = None

    if not args.insecure:
        cert_required = ssl.CERT_REQUIRED
    else:
        cert_required = ssl.CERT_NONE
        
    mqttc.tls_set(ca_certs=args.cacerts, certfile=None, keyfile=None, cert_reqs=cert_required, tls_version=tlsVersion)

    if args.insecure:
        mqttc.tls_insecure_set(True)

if args.username or args.password:
    mqttc.username_pw_set(args.username, args.password)

mqttc.on_message = on_message
mqttc.on_connect = on_connect
mqttc.on_publish = on_publish
mqttc.on_subscribe = on_subscribe

if args.debug:
    mqttc.on_log = on_log

print("Connecting to "+args.host+" port: "+str(port))
mqttc.connect(args.host, port, args.keepalive)
mqttc.subscribe(args.topic, args.qos)

mqttc.loop_forever()



#!/usr/bin/env python

# Test whether a client responds correctly to a PUBLISH with QoS 1.

# The client should connect to port 1888 with keepalive=60, clean session set,
# and client id publish-qos1-test
# The test will send a CONNACK message to the client with rc=0. Upon receiving
# the CONNACK the client should verify that rc==0.
# The test will send the client a PUBLISH message with topic
# "pub/qos1/receive", payload of "message", QoS=1 and mid=123. The client
# should handle this as per the spec by sending a PUBACK message.
# The client should then exit with return code==0.

import time

import context
import paho_test

rc = 1
keepalive = 60
connect_packet = paho_test.gen_connect("publish-qos1-test", keepalive=keepalive)
connack_packet = paho_test.gen_connack(rc=0)

disconnect_packet = paho_test.gen_disconnect()

mid = 123
publish_packet = paho_test.gen_publish(
    u"pub/qos1/receive", qos=1, mid=mid, payload="message".encode('utf-8'))
puback_packet = paho_test.gen_puback(mid)

sock = paho_test.create_server_socket()

client = context.start_client()

try:
    (conn, address) = sock.accept()
    conn.settimeout(10)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        conn.send(connack_packet)
        conn.send(publish_packet)

        if paho_test.expect_packet(conn, "puback", puback_packet):
            rc = 0

    conn.close()
finally:
    for i in range(0, 5):
        if client.returncode != None:
            break
        time.sleep(0.1)

    client.terminate()
    client.wait()
    sock.close()
    if client.returncode != 0:
        exit(1)

exit(rc)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2010-2013 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation
# Copyright (c) 2010,2011 Roger Light <roger@atchoo.org>
# All rights reserved.

# This shows a simple example of an MQTT subscriber.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.client as mqtt


def on_connect(mqttc, obj, flags, rc):
    print("rc: " + str(rc))


def on_message(mqttc, obj, msg):
    print(msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


def on_publish(mqttc, obj, mid):
    print("mid: " + str(mid))


def on_subscribe(mqttc, obj, mid, granted_qos):
    print("Subscribed: " + str(mid) + " " + str(granted_qos))


def on_log(mqttc, obj, level, string):
    print(string)


# If you want to use a specific client id, use
# mqttc = mqtt.Client("client-id")
# but note that the client id must be unique on the broker. Leaving the client
# id parameter empty will generate a random id for you.
mqttc = mqtt.Client()
mqttc.on_message = on_message
mqttc.on_connect = on_connect
mqttc.on_publish = on_publish
mqttc.on_subscribe = on_subscribe
# Uncomment to enable debug messages
# mqttc.on_log = on_log
mqttc.connect("mqtt.eclipse.org", 1883, 60)
mqttc.subscribe("$SYS/#", 0)

mqttc.loop_forever()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2013 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation

# This example shows how you can use the MQTT client in a class.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.client as mqtt

class MyMQTTClass(mqtt.Client):

    def on_connect(self, mqttc, obj, flags, rc):
        print("rc: "+str(rc))

    def on_message(self, mqttc, obj, msg):
        print(msg.topic+" "+str(msg.qos)+" "+str(msg.payload))

    def on_publish(self, mqttc, obj, mid):
        print("mid: "+str(mid))

    def on_subscribe(self, mqttc, obj, mid, granted_qos):
        print("Subscribed: "+str(mid)+" "+str(granted_qos))

    def on_log(self, mqttc, obj, level, string):
        print(string)

    def run(self):
        self.connect("mqtt.eclipse.org", 1883, 60)
        self.subscribe("$SYS/#", 0)

        rc = 0
        while rc == 0:
            rc = self.loop()
        return rc


# If you want to use a specific client id, use
# mqttc = MyMQTTClass("client-id")
# but note that the client id must be unique on the broker. Leaving the client
# id parameter empty will generate a random id for you.
mqttc = MyMQTTClass()
rc = mqttc.run()

print("rc: "+str(rc))



#!/usr/bin/env python

# Test whether a client produces a correct connect with a username and password.

# The client should connect to port 1888 with keepalive=60, clean session set,
# client id 01-unpwd-set, username set to uname and password set to ;'[08gn=#

import context
import paho_test

rc = 1
keepalive = 60
username = "uname"
password = ";'[08gn=#"
connect_packet = paho_test.gen_connect(
    "01-unpwd-set", keepalive=keepalive, username=username, password=password)

sock = paho_test.create_server_socket()

client = context.start_client()

try:
    (conn, address) = sock.accept()
    conn.settimeout(10)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        rc = 0

    conn.close()
finally:
    client.terminate()
    client.wait()
    sock.close()

exit(rc)



#!/usr/bin/env python

# Test whether a client sends a correct PUBLISH to a topic with QoS 1 and responds to a delay.

# The client should connect to port 1888 with keepalive=60, clean session set,
# and client id publish-qos1-test
# The test will send a CONNACK message to the client with rc=0. Upon receiving
# the CONNACK the client should verify that rc==0. If not, it should exit with
# return code=1.
# On a successful CONNACK, the client should send a PUBLISH message with topic
# "pub/qos1/test", payload "message" and QoS=1.
# The test will not respond to the first PUBLISH message, so the client must
# resend the PUBLISH message with dup=1. Note that to keep test durations low, a
# message retry timeout of less than 5 seconds is required for this test.
# On receiving the second PUBLISH message, the test will send the correct
# PUBACK response. On receiving the correct PUBACK response, the client should
# send a DISCONNECT message.

import context
import paho_test

rc = 1
keepalive = 60
connect_packet = paho_test.gen_connect("publish-qos1-test", keepalive=keepalive)
connack_packet = paho_test.gen_connack(rc=0)

disconnect_packet = paho_test.gen_disconnect()

mid = 1
publish_packet = paho_test.gen_publish(
    u"pub/qos1/test", qos=1, mid=mid, payload="message".encode('utf-8'))
publish_packet_dup = paho_test.gen_publish(
    u"pub/qos1/test", qos=1, mid=mid, payload="message".encode('utf-8'), dup=True)
puback_packet = paho_test.gen_puback(mid)

sock = paho_test.create_server_socket()

client = context.start_client()

try:
    (conn, address) = sock.accept()
    conn.settimeout(5)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        conn.send(connack_packet)

        if paho_test.expect_packet(conn, "publish", publish_packet):
            # Delay for > 3 seconds (message retry time)

            if paho_test.expect_packet(conn, "dup publish", publish_packet_dup):
                conn.send(puback_packet)

                if paho_test.expect_packet(conn, "disconnect", disconnect_packet):
                    rc = 0

    conn.close()
finally:
    client.terminate()
    client.wait()
    sock.close()

exit(rc)



"""
*******************************************************************
  Copyright (c) 2017, 2019 IBM Corp.

  All rights reserved. This program and the accompanying materials
  are made available under the terms of the Eclipse Public License v1.0
  and Eclipse Distribution License v1.0 which accompany this distribution.

  The Eclipse Public License is available at
     http://www.eclipse.org/legal/epl-v10.html
  and the Eclipse Distribution License is available at
    http://www.eclipse.org/org/documents/edl-v10.php.

  Contributors:
     Ian Craggs - initial implementation and/or documentation
*******************************************************************
"""

import sys


class MQTTException(Exception):
    pass


class SubscribeOptions(object):
    """The MQTT v5.0 subscribe options class.

    The options are:
        qos:                As in MQTT v3.1.1.
        noLocal:            True or False. If set to True, the subscriber will not receive its own publications.
        retainAsPublished:  True or False. If set to True, the retain flag on received publications will be as set
                            by the publisher.
        retainHandling:     RETAIN_SEND_ON_SUBSCRIBE, RETAIN_SEND_IF_NEW_SUB or RETAIN_DO_NOT_SEND
                            Controls when the broker should send retained messages:
                                - RETAIN_SEND_ON_SUBSCRIBE: on any successful subscribe request
                                - RETAIN_SEND_IF_NEW_SUB: only if the subscribe request is new
                                - RETAIN_DO_NOT_SEND: never send retained messages
    """

    # retain handling options
    RETAIN_SEND_ON_SUBSCRIBE, RETAIN_SEND_IF_NEW_SUB, RETAIN_DO_NOT_SEND = range(
        0, 3)

    def __init__(self, qos=0, noLocal=False, retainAsPublished=False, retainHandling=RETAIN_SEND_ON_SUBSCRIBE):
        """
        qos:                0, 1 or 2.  0 is the default.
        noLocal:            True or False. False is the default and corresponds to MQTT v3.1.1 behavior.
        retainAsPublished:  True or False. False is the default and corresponds to MQTT v3.1.1 behavior.
        retainHandling:     RETAIN_SEND_ON_SUBSCRIBE, RETAIN_SEND_IF_NEW_SUB or RETAIN_DO_NOT_SEND
                            RETAIN_SEND_ON_SUBSCRIBE is the default and corresponds to MQTT v3.1.1 behavior.
        """
        object.__setattr__(self, "names",
                           ["QoS", "noLocal", "retainAsPublished", "retainHandling"])
        self.QoS = qos  # bits 0,1
        self.noLocal = noLocal  # bit 2
        self.retainAsPublished = retainAsPublished  # bit 3
        self.retainHandling = retainHandling  # bits 4 and 5: 0, 1 or 2
        assert self.QoS in [0, 1, 2]
        assert self.retainHandling in [
            0, 1, 2], "Retain handling should be 0, 1 or 2"

    def __setattr__(self, name, value):
        if name not in self.names:
            raise MQTTException(
                name + " Attribute name must be one of "+str(self.names))
        object.__setattr__(self, name, value)

    def pack(self):
        assert self.QoS in [0, 1, 2]
        assert self.retainHandling in [
            0, 1, 2], "Retain handling should be 0, 1 or 2"
        noLocal = 1 if self.noLocal else 0
        retainAsPublished = 1 if self.retainAsPublished else 0
        data = [(self.retainHandling << 4) | (retainAsPublished << 3) |
                (noLocal << 2) | self.QoS]
        if sys.version_info[0] >= 3:
            buffer = bytes(data)
        else:
            buffer = bytearray(data)
        return buffer

    def unpack(self, buffer):
        b0 = buffer[0]
        self.retainHandling = ((b0 >> 4) & 0x03)
        self.retainAsPublished = True if ((b0 >> 3) & 0x01) == 1 else False
        self.noLocal = True if ((b0 >> 2) & 0x01) == 1 else False
        self.QoS = (b0 & 0x03)
        assert self.retainHandling in [
            0, 1, 2], "Retain handling should be 0, 1 or 2, not %d" % self.retainHandling
        assert self.QoS in [
            0, 1, 2], "QoS should be 0, 1 or 2, not %d" % self.QoS
        return 1

    def __repr__(self):
        return str(self)

    def __str__(self):
        return "{QoS="+str(self.QoS)+", noLocal="+str(self.noLocal) +\
            ", retainAsPublished="+str(self.retainAsPublished) +\
            ", retainHandling="+str(self.retainHandling)+"}"

    def json(self):
        data = {
            "QoS": self.QoS,
            "noLocal": self.noLocal,
            "retainAsPublished": self.retainAsPublished,
            "retainHandling": self.retainHandling,
        }
        return data



import base64
import re
import hashlib
from collections import OrderedDict

from six.moves import socketserver
import pytest
import paho.mqtt.client as client

from paho.mqtt.client import WebsocketConnectionError
from testsupport.broker import fake_websocket_broker


@pytest.fixture
def init_response_headers():
    # "Normal" websocket response from server
    response_headers = OrderedDict([
        ("Upgrade", "websocket"),
        ("Connection", "Upgrade"),
        ("Sec-WebSocket-Accept", "testwebsocketkey"),
        ("Sec-WebSocket-Protocol", "chat"),
    ])

    return response_headers


def get_websocket_response(response_headers):
    """ Takes headers and constructs HTTP response

    'HTTP/1.1 101 Switching Protocols' is the headers for the response,
    as expected in client.py
    """
    response = "\r\n".join([
        "HTTP/1.1 101 Switching Protocols",
        "\r\n".join("{}: {}".format(i, j) for i, j in response_headers.items()),
        "\r\n",
    ]).encode("utf8")

    return response


@pytest.mark.parametrize("proto_ver,proto_name", [
    (client.MQTTv31, "MQIsdp"),
    (client.MQTTv311, "MQTT"),
])
class TestInvalidWebsocketResponse(object):
    def test_unexpected_response(self, proto_ver, proto_name, fake_websocket_broker):
        """ Server responds with a valid code, but it's not what the client expected """

        mqttc = client.Client(
            "test_unexpected_response",
            protocol=proto_ver,
            transport="websockets"
            )

        class WebsocketHandler(socketserver.BaseRequestHandler):
            def handle(_self):
                # Respond with data passed in to serve()
                _self.request.sendall("200 OK".encode("utf8"))

        with fake_websocket_broker.serve(WebsocketHandler):
            with pytest.raises(WebsocketConnectionError) as exc:
                mqttc.connect("localhost", 1888, keepalive=10)

        assert str(exc.value) == "WebSocket handshake error"


@pytest.mark.parametrize("proto_ver,proto_name", [
    (client.MQTTv31, "MQIsdp"),
    (client.MQTTv311, "MQTT"),
])
class TestBadWebsocketHeaders(object):
    """ Testing for basic functionality in checking for headers """

    def _get_basic_handler(self, response_headers):
        """ Get a basic BaseRequestHandler which returns the information in
        self._response_headers
        """

        response = get_websocket_response(response_headers)

        class WebsocketHandler(socketserver.BaseRequestHandler):
            def handle(_self):
                self.data = _self.request.recv(1024).strip()
                print("Received '{:s}'".format(self.data.decode("utf8")))
                # Respond with data passed in to serve()
                _self.request.sendall(response)

        return WebsocketHandler

    def test_no_upgrade(self, proto_ver, proto_name, fake_websocket_broker,
                        init_response_headers):
        """ Server doesn't respond with 'connection: upgrade' """

        mqttc = client.Client(
            "test_no_upgrade",
            protocol=proto_ver,
            transport="websockets"
            )

        init_response_headers["Connection"] = "bad"
        response = self._get_basic_handler(init_response_headers)

        with fake_websocket_broker.serve(response):
            with pytest.raises(WebsocketConnectionError) as exc:
                mqttc.connect("localhost", 1888, keepalive=10)

        assert str(exc.value) == "WebSocket handshake error, connection not upgraded"

    def test_bad_secret_key(self, proto_ver, proto_name, fake_websocket_broker,
                            init_response_headers):
        """ Server doesn't give anything after connection: upgrade """

        mqttc = client.Client(
            "test_bad_secret_key",
            protocol=proto_ver,
            transport="websockets"
            )

        response = self._get_basic_handler(init_response_headers)

        with fake_websocket_broker.serve(response):
            with pytest.raises(WebsocketConnectionError) as exc:
                mqttc.connect("localhost", 1888, keepalive=10)

        assert str(exc.value) == "WebSocket handshake error, invalid secret key"


@pytest.mark.parametrize("proto_ver,proto_name", [
    (client.MQTTv31, "MQIsdp"),
    (client.MQTTv311, "MQTT"),
])
class TestValidHeaders(object):
    """ Testing for functionality in request/response headers """

    def _get_callback_handler(self, response_headers, check_request=None):
        """ Get a basic BaseRequestHandler which returns the information in
        self._response_headers
        """

        class WebsocketHandler(socketserver.BaseRequestHandler):
            def handle(_self):
                self.data = _self.request.recv(1024).strip()
                print("Received '{:s}'".format(self.data.decode("utf8")))

                decoded = self.data.decode("utf8")

                if check_request is not None:
                    check_request(decoded)

                # Create server hash
                GUID = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"
                key = re.search("sec-websocket-key: ([A-Za-z0-9+/=]*)", decoded, re.IGNORECASE).group(1)

                to_hash = "{:s}{:s}".format(key, GUID)
                hashed = hashlib.sha1(to_hash.encode("utf8"))
                encoded = base64.b64encode(hashed.digest()).decode("utf8")

                response_headers["Sec-WebSocket-Accept"] = encoded

                # Respond with the correct hash
                response = get_websocket_response(response_headers)

                _self.request.sendall(response)

        return WebsocketHandler

    def test_successful_connection(self, proto_ver, proto_name,
                                   fake_websocket_broker,
                                   init_response_headers):
        """ Connect successfully, on correct path """

        mqttc = client.Client(
            "test_successful_connection",
            protocol=proto_ver,
            transport="websockets"
            )

        response = self._get_callback_handler(init_response_headers)

        with fake_websocket_broker.serve(response):
            mqttc.connect("localhost", 1888, keepalive=10)

            mqttc.disconnect()

    @pytest.mark.parametrize("mqtt_path", [
        "/mqtt"
        "/special",
        None,
    ])
    def test_correct_path(self, proto_ver, proto_name, fake_websocket_broker,
                          mqtt_path, init_response_headers):
        """ Make sure it can connect on user specified paths """

        mqttc = client.Client(
            "test_correct_path",
            protocol=proto_ver,
            transport="websockets"
            )

        mqttc.ws_set_options(
            path=mqtt_path,
        )

        def check_path_correct(decoded):
            # Make sure it connects to the right path
            if mqtt_path:
                assert re.search("GET {:s} HTTP/1.1".format(mqtt_path), decoded, re.IGNORECASE) is not None

        response = self._get_callback_handler(
            init_response_headers,
            check_request=check_path_correct,
        )

        with fake_websocket_broker.serve(response):
            mqttc.connect("localhost", 1888, keepalive=10)

            mqttc.disconnect()

    @pytest.mark.parametrize("auth_headers", [
        {"Authorization": "test123"},
        {"Authorization": "test123", "auth2": "abcdef"},
        # Won't be checked, but make sure it still works even if the user passes it
        None,
    ])
    def test_correct_auth(self, proto_ver, proto_name, fake_websocket_broker,
                          auth_headers, init_response_headers):
        """ Make sure it sends the right auth headers """

        mqttc = client.Client(
            "test_correct_path",
            protocol=proto_ver,
            transport="websockets"
            )

        mqttc.ws_set_options(
            headers=auth_headers,
        )

        def check_headers_used(decoded):
            # Make sure it connects to the right path
            if auth_headers:
                for h in auth_headers:
                    assert re.search("{:s}: {:s}".format(h, auth_headers[h]), decoded, re.IGNORECASE) is not None

        response = self._get_callback_handler(
            init_response_headers,
            check_request=check_headers_used,
        )

        with fake_websocket_broker.serve(response):
            mqttc.connect("localhost", 1888, keepalive=10)

            mqttc.disconnect()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2013 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation
# Copyright (c) 2010,2011 Roger Light <roger@atchoo.org>
# All rights reserved.

# This shows an example of an MQTT client that clears all of the retained messages it receives.

import sys
import getopt

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.client as mqtt

final_mid = 0


def on_connect(mqttc, userdata, flags, rc):
    if userdata == True:
        print("rc: " + str(rc))


def on_message(mqttc, userdata, msg):
    global final_mid
    if msg.retain == 0:
        pass
        # sys.exit()
    else:
        if userdata == True:
            print("Clearing topic " + msg.topic)
        (rc, final_mid) = mqttc.publish(msg.topic, None, 1, True)


def on_publish(mqttc, userdata, mid):
    global final_mid
    if mid == final_mid:
        sys.exit()


def on_log(mqttc, userdata, level, string):
    print(string)


def print_usage():
    print(
        "mqtt_clear_retain.py [-d] [-h hostname] [-i clientid] [-k keepalive] [-p port] [-u username [-P password]] [-v] -t topic")


def main(argv):
    debug = False
    host = "mqtt.eclipse.org"
    client_id = None
    keepalive = 60
    port = 1883
    password = None
    topic = None
    username = None
    verbose = False

    try:
        opts, args = getopt.getopt(argv, "dh:i:k:p:P:t:u:v",
                                   ["debug", "id", "keepalive", "port", "password", "topic", "username", "verbose"])
    except getopt.GetoptError as s:
        print_usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-d", "--debug"):
            debug = True
        elif opt in ("-h", "--host"):
            host = arg
        elif opt in ("-i", "--id"):
            client_id = arg
        elif opt in ("-k", "--keepalive"):
            keepalive = int(arg)
        elif opt in ("-p", "--port"):
            port = int(arg)
        elif opt in ("-P", "--password"):
            password = arg
        elif opt in ("-t", "--topic"):
            topic = arg
            print(topic)
        elif opt in ("-u", "--username"):
            username = arg
        elif opt in ("-v", "--verbose"):
            verbose = True

    if topic == None:
        print("You must provide a topic to clear.\n")
        print_usage()
        sys.exit(2)

    mqttc = mqtt.Client(client_id)
    mqttc._userdata = verbose
    mqttc.on_message = on_message
    mqttc.on_publish = on_publish
    mqttc.on_connect = on_connect
    if debug:
        mqttc.on_log = on_log

    if username:
        mqttc.username_pw_set(username, password)
    mqttc.connect(host, port, keepalive)
    mqttc.subscribe(topic)
    mqttc.loop_forever()


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation
# All rights reserved.

# This shows a simple example of an MQTT subscriber using a per-subscription message handler.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.client as mqtt


def on_message_msgs(mosq, obj, msg):
    # This callback will only be called for messages with topics that match
    # $SYS/broker/messages/#
    print("MESSAGES: " + msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


def on_message_bytes(mosq, obj, msg):
    # This callback will only be called for messages with topics that match
    # $SYS/broker/bytes/#
    print("BYTES: " + msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


def on_message(mosq, obj, msg):
    # This callback will be called for messages that we receive that do not
    # match any patterns defined in topic specific callbacks, i.e. in this case
    # those messages that do not have topics $SYS/broker/messages/# nor
    # $SYS/broker/bytes/#
    print(msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


mqttc = mqtt.Client()

# Add message callbacks that will only trigger on a specific subscription match.
mqttc.message_callback_add("$SYS/broker/messages/#", on_message_msgs)
mqttc.message_callback_add("$SYS/broker/bytes/#", on_message_bytes)
mqttc.on_message = on_message
mqttc.connect("mqtt.eclipse.org", 1883, 60)
mqttc.subscribe("$SYS/#", 0)

mqttc.loop_forever()



import hmac
import hashlib
import datetime
import base64
import uuid
import functools

from paho.mqtt.client import Client


def get_amazon_auth_headers(access_key, secret_key, region, host, port, headers=None):
    """ Get the amazon auth headers for working with the amazon websockets
    protocol

    Requires a lot of extra stuff:

    http://docs.aws.amazon.com/general/latest/gr//sigv4-create-canonical-request.html
    http://docs.aws.amazon.com/general/latest/gr//signature-v4-examples.html#signature-v4-examples-pythonw
    http://docs.aws.amazon.com/general/latest/gr//sigv4-signed-request-examples.html#sig-v4-examples-get-auth-header

    Args:
        access_key (str): Amazon access key (AWS_ACCESS_KEY_ID)
        secret_key (str): Amazon secret access key (AWS_SECRET_ACCESS_KEY)
        region (str): aws region
        host (str): iot endpoint (xxxxxxxxxxxxxx.iot.<region>.amazonaws.com)
        headers (dict): a dictionary of the original headers- normally websocket headers

    Returns:
        dict: A string containing the headers that amazon expects in the auth
            request for the iot websocket service
    """

    # pylint: disable=unused-variable,unused-argument

    def sign(key, msg):
        return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

    def getSignatureKey(key, dateStamp, regionName, serviceName):
        kDate = sign(("AWS4" + key).encode("utf-8"), dateStamp)
        kRegion = sign(kDate, regionName)
        kService = sign(kRegion, serviceName)
        kSigning = sign(kService, "aws4_request")
        return kSigning

    service = "iotdevicegateway"
    algorithm = "AWS4-HMAC-SHA256"

    t = datetime.datetime.utcnow()
    amzdate = t.strftime('%Y%m%dT%H%M%SZ')
    datestamp = t.strftime("%Y%m%d") # Date w/o time, used in credential scope

    if headers is None:
        headers = {
            "Host": "{0:s}:443".format(host),
            "Upgrade": "websocket",
            "Connection": "Upgrade",
            "Origin": "https://{0:s}:443".format(host),
            "Sec-WebSocket-Key": base64.b64encode(uuid.uuid4().bytes),
            "Sec-Websocket-Version": "13",
            "Sec-Websocket-Protocol": "mqtt",
        }

    headers.update({
        "X-Amz-Date": amzdate,
    })

    # get into 'canonical' form - lowercase, sorted alphabetically
    canonical_headers = "\n".join(sorted("{}:{}".format(i.lower(), j).strip() for i, j in headers.items()))
    # Headers to sign - alphabetical order
    signed_headers = ";".join(sorted(i.lower().strip() for i in headers.keys()))

    # No payload
    payload_hash = hashlib.sha256("").hexdigest().lower()

    request_parts = [
        "GET",
        "/mqtt",
        # no query parameters
        "",
        canonical_headers + "\n",
        signed_headers,
        payload_hash,
    ]

    canonical_request = "\n".join(request_parts)

    # now actually hash request and sign
    hashed_request = hashlib.sha256(canonical_request).hexdigest()

    credential_scope = "{datestamp:s}/{region:s}/{service:s}/aws4_request".format(**locals())
    string_to_sign = "{algorithm:s}\n{amzdate:s}\n{credential_scope:s}\n{hashed_request:s}".format(**locals())

    signing_key = getSignatureKey(secret_key, datestamp, region, service)
    signature = hmac.new(signing_key, (string_to_sign).encode('utf-8'), hashlib.sha256).hexdigest()

    # create auth header
    authorization_header = "{algorithm:s} Credential={access_key:s}/{credential_scope:s}, SignedHeaders={signed_headers:s}, Signature={signature:s}".format(**locals())

    # get final header string
    headers["Authorization"] = authorization_header

    return headers


def example_use():
    access_key = os.environ["AWS_ACCESS_KEY_ID"]
    secret_key = os.environ["AWS_SECRET_ACCESS_KEY"]
    port = 8883

    region = "eu-west-1"

    # This is specific to your AWS account
    host = "abc123def456.iot.{0:s}.amazonaws.com".format(region)

    extra_headers = functools.partial(
        get_amazon_auth_headers,
        access_key,
        secret_key,
        region,
        host,
        port,
    )

    client = Client(transport="websockets")

    client.ws_set_options(headers=extra_headers)

    # Use client as normal from here



"""
*******************************************************************
  Copyright (c) 2017, 2019 IBM Corp.

  All rights reserved. This program and the accompanying materials
  are made available under the terms of the Eclipse Public License v1.0
  and Eclipse Distribution License v1.0 which accompany this distribution.

  The Eclipse Public License is available at
     http://www.eclipse.org/legal/epl-v10.html
  and the Eclipse Distribution License is available at
    http://www.eclipse.org/org/documents/edl-v10.php.

  Contributors:
     Ian Craggs - initial implementation and/or documentation
*******************************************************************
"""

import sys
from .packettypes import PacketTypes


class ReasonCodes:
    """MQTT version 5.0 reason codes class.

    See ReasonCodes.names for a list of possible numeric values along with their 
    names and the packets to which they apply.

    """

    def __init__(self, packetType, aName="Success", identifier=-1):
        """
        packetType: the type of the packet, such as PacketTypes.CONNECT that
            this reason code will be used with.  Some reason codes have different
            names for the same identifier when used a different packet type.

        aName: the String name of the reason code to be created.  Ignored
            if the identifier is set.

        identifier: an integer value of the reason code to be created.  

        """

        self.packetType = packetType
        self.names = {
            0: {"Success": [PacketTypes.CONNACK, PacketTypes.PUBACK,
                            PacketTypes.PUBREC, PacketTypes.PUBREL, PacketTypes.PUBCOMP,
                            PacketTypes.UNSUBACK, PacketTypes.AUTH],
                "Normal disconnection": [PacketTypes.DISCONNECT],
                "Granted QoS 0": [PacketTypes.SUBACK]},
            1: {"Granted QoS 1": [PacketTypes.SUBACK]},
            2: {"Granted QoS 2": [PacketTypes.SUBACK]},
            4: {"Disconnect with will message": [PacketTypes.DISCONNECT]},
            16: {"No matching subscribers":
                 [PacketTypes.PUBACK, PacketTypes.PUBREC]},
            17: {"No subscription found": [PacketTypes.UNSUBACK]},
            24: {"Continue authentication": [PacketTypes.AUTH]},
            25: {"Re-authenticate": [PacketTypes.AUTH]},
            128: {"Unspecified error": [PacketTypes.CONNACK, PacketTypes.PUBACK,
                                        PacketTypes.PUBREC, PacketTypes.SUBACK, PacketTypes.UNSUBACK,
                                        PacketTypes.DISCONNECT], },
            129: {"Malformed packet":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            130: {"Protocol error":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            131: {"Implementation specific error": [PacketTypes.CONNACK,
                                                    PacketTypes.PUBACK, PacketTypes.PUBREC, PacketTypes.SUBACK,
                                                    PacketTypes.UNSUBACK, PacketTypes.DISCONNECT], },
            132: {"Unsupported protocol version": [PacketTypes.CONNACK]},
            133: {"Client identifier not valid": [PacketTypes.CONNACK]},
            134: {"Bad user name or password": [PacketTypes.CONNACK]},
            135: {"Not authorized": [PacketTypes.CONNACK, PacketTypes.PUBACK,
                                     PacketTypes.PUBREC, PacketTypes.SUBACK, PacketTypes.UNSUBACK,
                                     PacketTypes.DISCONNECT], },
            136: {"Server unavailable": [PacketTypes.CONNACK]},
            137: {"Server busy": [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            138: {"Banned": [PacketTypes.CONNACK]},
            139: {"Server shutting down": [PacketTypes.DISCONNECT]},
            140: {"Bad authentication method":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            141: {"Keep alive timeout": [PacketTypes.DISCONNECT]},
            142: {"Session taken over": [PacketTypes.DISCONNECT]},
            143: {"Topic filter invalid":
                  [PacketTypes.SUBACK, PacketTypes.UNSUBACK, PacketTypes.DISCONNECT]},
            144: {"Topic name invalid":
                  [PacketTypes.CONNACK, PacketTypes.PUBACK,
                   PacketTypes.PUBREC, PacketTypes.DISCONNECT]},
            145: {"Packet identifier in use":
                  [PacketTypes.PUBACK, PacketTypes.PUBREC,
                   PacketTypes.SUBACK, PacketTypes.UNSUBACK]},
            146: {"Packet identifier not found":
                  [PacketTypes.PUBREL, PacketTypes.PUBCOMP]},
            147: {"Receive maximum exceeded": [PacketTypes.DISCONNECT]},
            148: {"Topic alias invalid": [PacketTypes.DISCONNECT]},
            149: {"Packet too large": [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            150: {"Message rate too high": [PacketTypes.DISCONNECT]},
            151: {"Quota exceeded": [PacketTypes.CONNACK, PacketTypes.PUBACK,
                                     PacketTypes.PUBREC, PacketTypes.SUBACK, PacketTypes.DISCONNECT], },
            152: {"Administrative action": [PacketTypes.DISCONNECT]},
            153: {"Payload format invalid":
                  [PacketTypes.PUBACK, PacketTypes.PUBREC, PacketTypes.DISCONNECT]},
            154: {"Retain not supported":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            155: {"QoS not supported":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            156: {"Use another server":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            157: {"Server moved":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            158: {"Shared subscription not supported":
                  [PacketTypes.SUBACK, PacketTypes.DISCONNECT]},
            159: {"Connection rate exceeded":
                  [PacketTypes.CONNACK, PacketTypes.DISCONNECT]},
            160: {"Maximum connect time":
                  [PacketTypes.DISCONNECT]},
            161: {"Subscription identifiers not supported":
                  [PacketTypes.SUBACK, PacketTypes.DISCONNECT]},
            162: {"Wildcard subscription not supported":
                  [PacketTypes.SUBACK, PacketTypes.DISCONNECT]},
        }
        if identifier == -1:
            if packetType == PacketTypes.DISCONNECT and aName == "Success":
                  aName = "Normal disconnection"
            self.set(aName)
        else:
            self.value = identifier
            self.getName()  # check it's good

    def __getName__(self, packetType, identifier):
        """
        Get the reason code string name for a specific identifier.
        The name can vary by packet type for the same identifier, which
        is why the packet type is also required.

        Used when displaying the reason code.
        """
        assert identifier in self.names.keys(), identifier
        names = self.names[identifier]
        namelist = [name for name in names.keys() if packetType in names[name]]
        assert len(namelist) == 1
        return namelist[0]

    def getId(self, name):
        """
        Get the numeric id corresponding to a reason code name.

        Used when setting the reason code for a packetType
        check that only valid codes for the packet are set.
        """
        identifier = None
        for code in self.names.keys():
            if name in self.names[code].keys():
                if self.packetType in self.names[code][name]:
                    identifier = code
                break
        assert identifier != None, name
        return identifier

    def set(self, name):
        self.value = self.getId(name)

    def unpack(self, buffer):
        c = buffer[0]
        if sys.version_info[0] < 3:
            c = ord(c)
        name = self.__getName__(self.packetType, c)
        self.value = self.getId(name)
        return 1

    def getName(self):
        """Returns the reason code name corresponding to the numeric value which is set.
        """
        return self.__getName__(self.packetType, self.value)

    def __eq__(self, other):
        if isinstance(other, int):
            return self.value == other
        if isinstance(other, str):
            return self.value == str(self)
        if isinstance(other, ReasonCodes):
            return self.value == other.value
        return False

    def __str__(self):
        return self.getName()

    def json(self):
        return self.getName()

    def pack(self):
        return bytearray([self.value])


# -*- coding: utf-8 -*-

import sys
import os
import subprocess

try:
    import ssl
except ImportError:
    ssl = None

# Ensure can import paho_test package
try:
    import paho_test

except ImportError:
    # This part is only required when paho_test module is not on Python path
    # From http://stackoverflow.com/questions/279237/python-import-a-module-from-a-folder
    import inspect

    cmd_subfolder = os.path.realpath(
        os.path.abspath(
            os.path.join(
                os.path.split(
                    inspect.getfile(inspect.currentframe())
                )[0],
                "..",
            )
        )
    )
    if cmd_subfolder not in sys.path:
        sys.path.insert(0, cmd_subfolder)

    import paho_test

env = dict(os.environ)
pp = env.get('PYTHONPATH', '')
env['PYTHONPATH'] = '../../src' + os.pathsep + pp


def start_client():
    args = [sys.executable, ] + sys.argv[1:]
    client = subprocess.Popen(args, env=env)
    return client


def check_ssl():
    if ssl is None:
        print("WARNING: SSL not available in current environment")
        exit(0)

    if not hasattr(ssl, 'SSLContext'):
        print("WARNING: SSL without SSLContext is not supported")
        exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2016 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation

# This shows an example of using the subscribe.simple helper function.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.subscribe as subscribe

topics = ['#']

m = subscribe.simple(topics, hostname="mqtt.eclipse.org", retained=False, msg_count=2)
for a in m:
    print(a.topic)
    print(a.payload)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation

# This shows an example of using the publish.single helper function with unicode topic and payload.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.publish as publish

topic = u"paho/test/single/ô"
payload = u"bôô"
publish.single(topic, payload, hostname="mqtt.eclipse.org")



"""
*******************************************************************
  Copyright (c) 2013, 2019 IBM Corp.

  All rights reserved. This program and the accompanying materials
  are made available under the terms of the Eclipse Public License v1.0
  and Eclipse Distribution License v1.0 which accompany this distribution.

  The Eclipse Public License is available at
     http://www.eclipse.org/legal/epl-v10.html
  and the Eclipse Distribution License is available at
    http://www.eclipse.org/org/documents/edl-v10.php.

  Contributors:
     Ian Craggs - initial implementation and/or documentation
*******************************************************************
"""

import unittest
import time
import threading
import getopt
import sys
import logging
import traceback

import paho.mqtt
from paho.mqtt.properties import Properties
from paho.mqtt.reasoncodes import ReasonCodes
from paho.mqtt.subscribeoptions import SubscribeOptions
from paho.mqtt.packettypes import PacketTypes
import paho.mqtt.client


class Callbacks:

    def __init__(self):
        self.messages = []
        self.publisheds = []
        self.subscribeds = []
        self.unsubscribeds = []
        self.disconnecteds = []
        self.connecteds = []

    def __str__(self):
        return str(self.messages) + str(self.messagedicts) + str(self.publisheds) + \
            str(self.subscribeds) + \
            str(self.unsubscribeds) + str(self.disconnects)

    def clear(self):
        self.__init__()

    def on_connect(self, client, userdata, flags, reasonCode, properties):
        self.connecteds.append({"userdata": userdata, "flags": flags,
                                "reasonCode": reasonCode, "properties": properties})

    def wait(self, alist, timeout=2):
        interval = 0.2
        total = 0
        while len(alist) == 0 and total < timeout:
            time.sleep(interval)
            total += interval
        return alist.pop(0)  # if len(alist) > 0 else None

    def wait_connected(self):
        return self.wait(self.connecteds)

    def on_disconnect(self, client, userdata, reasonCode, properties=None):
        self.disconnecteds.append(
            {"reasonCode": reasonCode, "properties": properties})

    def wait_disconnected(self):
        return self.wait(self.disconnecteds)

    def on_message(self, client, userdata, message):
        self.messages.append({"userdata": userdata, "message": message})

    def published(self, client, userdata, msgid):
        self.publisheds.append(msgid)

    def wait_published(self):
        return self.wait(self.publisheds)

    def on_subscribe(self, client, userdata, mid, reasonCodes, properties):
        self.subscribeds.append({"mid": mid, "userdata": userdata,
                                 "properties": properties, "reasonCodes": reasonCodes})

    def wait_subscribed(self):
        return self.wait(self.subscribeds)

    def unsubscribed(self, client, userdata, mid, properties, reasonCodes):
        self.unsubscribeds.append({"mid": mid, "userdata": userdata,
                                   "properties": properties, "reasonCodes": reasonCodes})

    def wait_unsubscribed(self):
        return self.wait(self.unsubscribeds)

    def on_log(self, client, userdata, level, buf):
        print(buf)

    def register(self, client):
        client.on_connect = self.on_connect
        client.on_subscribe = self.on_subscribe
        client.on_publish = self.published
        client.on_unsubscribe = self.unsubscribed
        client.on_message = self.on_message
        client.on_disconnect = self.on_disconnect
        client.on_log = self.on_log


def cleanRetained(port):
    callback = Callbacks()
    curclient = paho.mqtt.client.Client("clean retained".encode("utf-8"),
                                        protocol=paho.mqtt.client.MQTTv5)
    curclient.loop_start()
    callback.register(curclient)
    curclient.connect(host="localhost", port=port)
    response = callback.wait_connected()
    curclient.subscribe("#", options=SubscribeOptions(qos=0))
    response = callback.wait_subscribed()  # wait for retained messages to arrive
    time.sleep(1)
    for message in callback.messages:
        logging.info("deleting retained message for topic", message["message"])
        curclient.publish(message["message"].topic, b"", 0, retain=True)
    curclient.disconnect()
    curclient.loop_stop()
    time.sleep(.1)


def cleanup(port):
    # clean all client state
    print("clean up starting")
    clientids = ("aclient", "bclient")

    for clientid in clientids:
        curclient = paho.mqtt.client.Client(clientid.encode(
            "utf-8"), protocol=paho.mqtt.client.MQTTv5)
        curclient.loop_start()
        curclient.connect(host="localhost", port=port, clean_start=True)
        time.sleep(.1)
        curclient.disconnect()
        time.sleep(.1)
        curclient.loop_stop()

    # clean retained messages
    cleanRetained(port)
    print("clean up finished")


class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        global callback, callback2, aclient, bclient

        sys.path.append("paho.mqtt.testing/interoperability/")
        try:
            import mqtt.brokers
        except ImportError:
            raise unittest.SkipTest("paho.mqtt.testing not present.")

        cls._test_broker = threading.Thread(
            target=mqtt.brokers.run,
            kwargs={
                "config": ["listener 0"],
            },
        )
        cls._test_broker.daemon = True
        cls._test_broker.start()
        # Wait a bit for TCP server to bind to an address
        time.sleep(0.5)
        # Hack to find the port used by the test broker...
        cls._test_broker_port = mqtt.brokers.listeners.TCPListeners.server.socket.getsockname()[1]
        setData()
        cleanup(cls._test_broker_port)

        callback = Callbacks()
        callback2 = Callbacks()

        #aclient = mqtt_client.Client(b"\xEF\xBB\xBF" + "myclientid".encode("utf-8"))
        #aclient = mqtt_client.Client("myclientid".encode("utf-8"))
        aclient = paho.mqtt.client.Client("aclient".encode(
            "utf-8"), protocol=paho.mqtt.client.MQTTv5)
        callback.register(aclient)

        bclient = paho.mqtt.client.Client("bclient".encode(
            "utf-8"), protocol=paho.mqtt.client.MQTTv5)
        callback2.register(bclient)

    @classmethod
    def tearDownClass(cls):
        # Another hack to stop the test broker... we rely on fact that it use a sockserver.TCPServer
        import mqtt.brokers
        mqtt.brokers.listeners.TCPListeners.server.shutdown()
        cls._test_broker.join(5)

    def waitfor(self, queue, depth, limit):
        total = 0
        while len(queue) < depth and total < limit:
            interval = .5
            total += interval
            time.sleep(interval)

    def test_basic(self):
        aclient.connect(host="localhost", port=self._test_broker_port)
        aclient.loop_start()
        response = callback.wait_connected()
        self.assertEqual(response["reasonCode"].getName(), "Success")

        aclient.subscribe(topics[0], options=SubscribeOptions(qos=2))
        response = callback.wait_subscribed()
        self.assertEqual(response["reasonCodes"][0].getName(), "Granted QoS 2")

        aclient.publish(topics[0], b"qos 0")
        aclient.publish(topics[0], b"qos 1", 1)
        aclient.publish(topics[0], b"qos 2", 2)
        i = 0
        while len(callback.messages) < 3 and i < 10:
            time.sleep(.2)
            i += 1
        self.assertEqual(len(callback.messages), 3)
        aclient.disconnect()

        callback.clear()
        aclient.loop_stop()

    def test_retained_message(self):
        qos0topic = "fromb/qos 0"
        qos1topic = "fromb/qos 1"
        qos2topic = "fromb/qos2"
        wildcardtopic = "fromb/+"

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.UserProperty = ("a", "2")
        publish_properties.UserProperty = ("c", "3")

        # retained messages
        callback.clear()
        aclient.connect(host="localhost", port=self._test_broker_port)
        aclient.loop_start()
        response = callback.wait_connected()
        aclient.publish(topics[1], b"qos 0", 0,
                        retain=True, properties=publish_properties)
        aclient.publish(topics[2], b"qos 1", 1,
                        retain=True, properties=publish_properties)
        aclient.publish(topics[3], b"qos 2", 2,
                        retain=True, properties=publish_properties)
        # wait until those messages are published
        time.sleep(1)
        aclient.subscribe(wildtopics[5], options=SubscribeOptions(qos=2))
        response = callback.wait_subscribed()
        self.assertEqual(response["reasonCodes"][0].getName(), "Granted QoS 2")

        time.sleep(1)
        aclient.disconnect()
        aclient.loop_stop()

        self.assertEqual(len(callback.messages), 3)
        userprops = callback.messages[0]["message"].properties.UserProperty
        self.assertTrue(userprops in [[("a", "2"), ("c", "3")], [
                        ("c", "3"), ("a", "2")]], userprops)
        userprops = callback.messages[1]["message"].properties.UserProperty
        self.assertTrue(userprops in [[("a", "2"), ("c", "3")], [
                        ("c", "3"), ("a", "2")]], userprops)
        userprops = callback.messages[2]["message"].properties.UserProperty
        self.assertTrue(userprops in [[("a", "2"), ("c", "3")], [
                        ("c", "3"), ("a", "2")]], userprops)
        qoss = [callback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)

        cleanRetained(self._test_broker_port)

    def test_will_message(self):
        # will messages and keep alive
        callback.clear()
        callback2.clear()
        self.assertEqual(len(callback2.messages), 0, callback2.messages)

        will_properties = Properties(PacketTypes.WILLMESSAGE)
        will_properties.WillDelayInterval = 0  # this is the default anyway
        will_properties.UserProperty = ("a", "2")
        will_properties.UserProperty = ("c", "3")

        aclient.will_set(topics[2], payload=b"will message",
                         properties=will_properties)

        aclient.connect(host="localhost", port=self._test_broker_port, keepalive=2)
        aclient.loop_start()
        response = callback.wait_connected()
        bclient.connect(host="localhost", port=self._test_broker_port)
        bclient.loop_start()
        response = callback2.wait_connected()
        bclient.subscribe(topics[2], qos=2)
        response = callback2.wait_subscribed()
        self.assertEqual(response["reasonCodes"][0].getName(), "Granted QoS 2")

        # keep alive timeout ought to be triggered so the will message is received
        aclient.loop_stop()  # so that pings aren't sent
        self.waitfor(callback2.messages, 1, 10)
        bclient.disconnect()
        bclient.loop_stop()
        # should have the will message
        self.assertEqual(len(callback2.messages), 1, callback2.messages)
        props = callback2.messages[0]["message"].properties
        self.assertEqual(props.UserProperty, [("a", "2"), ("c", "3")])

    def test_zero_length_clientid(self):
        logging.info("Zero length clientid test starting")

        callback0 = Callbacks()

        client0 = paho.mqtt.client.Client(protocol=paho.mqtt.client.MQTTv5)
        callback0.register(client0)
        client0.loop_start()
        # should not be rejected
        client0.connect(host="localhost", port=self._test_broker_port, clean_start=False)
        response = callback0.wait_connected()
        self.assertEqual(response["reasonCode"].getName(), "Success")
        self.assertTrue(
            len(response["properties"].AssignedClientIdentifier) > 0)
        client0.disconnect()
        client0.loop_stop()

        client0 = paho.mqtt.client.Client(protocol=paho.mqtt.client.MQTTv5)
        callback0.register(client0)
        client0.loop_start()
        client0.connect(host="localhost", port=self._test_broker_port)  # should work
        response = callback0.wait_connected()
        self.assertEqual(response["reasonCode"].getName(), "Success")
        self.assertTrue(
            len(response["properties"].AssignedClientIdentifier) > 0)
        client0.disconnect()
        client0.loop_stop()

        # when we supply a client id, we should not get one assigned
        client0 = paho.mqtt.client.Client(
            "client0", protocol=paho.mqtt.client.MQTTv5)
        callback0.register(client0)
        client0.loop_start()
        client0.connect(host="localhost", port=self._test_broker_port)  # should work
        response = callback0.wait_connected()
        self.assertEqual(response["reasonCode"].getName(), "Success")
        self.assertFalse(
            hasattr(response["properties"], "AssignedClientIdentifier"))
        client0.disconnect()
        client0.loop_stop()

    def test_offline_message_queueing(self):
        # message queueing for offline clients
        cleanRetained(self._test_broker_port)
        ocallback = Callbacks()
        clientid = "offline message queueing".encode("utf-8")

        oclient = paho.mqtt.client.Client(
            clientid, protocol=paho.mqtt.client.MQTTv5)
        ocallback.register(oclient)
        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.SessionExpiryInterval = 99999
        oclient.loop_start()
        oclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        response = ocallback.wait_connected()
        oclient.subscribe(wildtopics[5], qos=2)
        response = ocallback.wait_subscribed()
        oclient.disconnect()
        oclient.loop_stop()

        bclient.loop_start()
        bclient.connect(host="localhost", port=self._test_broker_port)
        response = callback2.wait_connected()
        bclient.publish(topics[1], b"qos 0", 0)
        bclient.publish(topics[2], b"qos 1", 1)
        bclient.publish(topics[3], b"qos 2", 2)
        time.sleep(2)
        bclient.disconnect()
        bclient.loop_stop()

        oclient = paho.mqtt.client.Client(
            clientid, protocol=paho.mqtt.client.MQTTv5)
        ocallback.register(oclient)
        oclient.loop_start()
        oclient.connect(host="localhost", port=self._test_broker_port, clean_start=False)
        response = ocallback.wait_connected()
        time.sleep(2)
        oclient.disconnect()
        oclient.loop_stop()

        self.assertTrue(len(ocallback.messages) in [
                        2, 3], len(ocallback.messages))
        logging.info("This server %s queueing QoS 0 messages for offline clients" %
                     ("is" if len(ocallback.messages) == 3 else "is not"))

    def test_overlapping_subscriptions(self):
        # overlapping subscriptions. When there is more than one matching subscription for the same client for a topic,
        # the server may send back one message with the highest QoS of any matching subscription, or one message for
        # each subscription with a matching QoS.
        ocallback = Callbacks()
        clientid = "overlapping subscriptions".encode("utf-8")

        oclient = paho.mqtt.client.Client(
            clientid, protocol=paho.mqtt.client.MQTTv5)
        ocallback.register(oclient)

        oclient.loop_start()
        oclient.connect(host="localhost", port=self._test_broker_port)
        ocallback.wait_connected()
        oclient.subscribe([(wildtopics[6], SubscribeOptions(qos=2)),
                           (wildtopics[0], SubscribeOptions(qos=1))])
        ocallback.wait_subscribed()
        oclient.publish(topics[3], b"overlapping topic filters", 2)
        ocallback.wait_published()
        time.sleep(1)
        self.assertTrue(len(ocallback.messages) in [1, 2], ocallback.messages)
        if len(ocallback.messages) == 1:
            logging.info(
                "This server is publishing one message for all matching overlapping subscriptions, not one for each.")
            self.assertEqual(
                ocallback.messages[0]["message"].qos, 2, ocallback.messages[0]["message"].qos)
        else:
            logging.info(
                "This server is publishing one message per each matching overlapping subscription.")
            self.assertTrue((ocallback.messages[0]["message"].qos == 2 and ocallback.messages[1]["message"].qos == 1) or
                            (ocallback.messages[0]["message"].qos == 1 and ocallback.messages[1]["message"].qos == 2), callback.messages)
        oclient.disconnect()
        oclient.loop_stop()
        ocallback.clear()

    def test_subscribe_failure(self):
        # Subscribe failure.  A new feature of MQTT 3.1.1 is the ability to send back negative reponses to subscribe
        # requests.  One way of doing this is to subscribe to a topic which is not allowed to be subscribed to.
        logging.info("Subscribe failure test starting")

        ocallback = Callbacks()
        clientid = "subscribe failure".encode("utf-8")
        oclient = paho.mqtt.client.Client(
            clientid, protocol=paho.mqtt.client.MQTTv5)
        ocallback.register(oclient)
        oclient.loop_start()
        oclient.connect(host="localhost", port=self._test_broker_port)
        ocallback.wait_connected()
        oclient.subscribe(nosubscribe_topics[0], qos=2)
        response = ocallback.wait_subscribed()

        self.assertEqual(response["reasonCodes"][0].getName(), "Unspecified error",
                         "return code should be 0x80 %s" % response["reasonCodes"][0].getName())
        oclient.disconnect()
        oclient.loop_stop()

    def test_unsubscribe(self):
        callback2.clear()
        bclient.connect(host="localhost", port=self._test_broker_port)
        bclient.loop_start()
        callback2.wait_connected()
        bclient.subscribe(topics[0], qos=2)
        callback2.wait_subscribed()
        bclient.subscribe(topics[1], qos=2)
        callback2.wait_subscribed()
        bclient.subscribe(topics[2], qos=2)
        callback2.wait_subscribed()
        time.sleep(1)  # wait for any retained messages, hopefully
        # Unsubscribe from one topic
        bclient.unsubscribe(topics[0])
        callback2.wait_unsubscribed()
        callback2.clear()  # if there were any retained messsages

        aclient.connect(host="localhost", port=self._test_broker_port)
        aclient.loop_start()
        callback.wait_connected()
        aclient.publish(topics[0], b"topic 0 - unsubscribed", 1, retain=False)
        aclient.publish(topics[1], b"topic 1", 1, retain=False)
        aclient.publish(topics[2], b"topic 2", 1, retain=False)
        time.sleep(2)

        bclient.disconnect()
        bclient.loop_stop()
        aclient.disconnect()
        aclient.loop_stop()
        self.assertEqual(len(callback2.messages), 2, callback2.messages)

    def new_client(self, clientid):
        callback = Callbacks()
        client = paho.mqtt.client.Client(clientid.encode(
            "utf-8"), protocol=paho.mqtt.client.MQTTv5)
        callback.register(client)
        client.loop_start()
        return client, callback

    def test_session_expiry(self):
        # no session expiry property == never expire

        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.SessionExpiryInterval = 0  # expire immediately

        clientid = "session expiry"

        eclient, ecallback = self.new_client(clientid)

        eclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = ecallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        eclient.subscribe(topics[0], qos=2)
        ecallback.wait_subscribed()
        eclient.disconnect()
        ecallback.wait_disconnected()
        eclient.loop_stop()

        fclient, fcallback = self.new_client(clientid)

        # session should immediately expire
        fclient.connect_async(host="localhost", port=self._test_broker_port, clean_start=False,
                              properties=connect_properties)
        connack = fcallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        fclient.disconnect()
        fcallback.wait_disconnected()

        connect_properties.SessionExpiryInterval = 5

        eclient, ecallback = self.new_client(clientid)

        eclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = ecallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        eclient.subscribe(topics[0], qos=2)
        ecallback.wait_subscribed()
        eclient.disconnect()
        ecallback.wait_disconnected()
        eclient.loop_stop()

        time.sleep(2)
        # session should still exist
        fclient, fcallback = self.new_client(clientid)
        fclient.connect(host="localhost", port=self._test_broker_port, clean_start=False,
                        properties=connect_properties)
        connack = fcallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], True)
        fclient.disconnect()
        fcallback.wait_disconnected()
        fclient.loop_stop()

        time.sleep(6)
        # session should not exist
        fclient, fcallback = self.new_client(clientid)
        fclient.connect(host="localhost", port=self._test_broker_port, clean_start=False,
                        properties=connect_properties)
        connack = fcallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        fclient.disconnect()
        fcallback.wait_disconnected()
        fclient.loop_stop()

        eclient, ecallback = self.new_client(clientid)
        connect_properties.SessionExpiryInterval = 1
        connack = eclient.connect(
            host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = ecallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        eclient.subscribe(topics[0], qos=2)
        ecallback.wait_subscribed()
        disconnect_properties = Properties(PacketTypes.DISCONNECT)
        disconnect_properties.SessionExpiryInterval = 5
        eclient.disconnect(properties=disconnect_properties)
        ecallback.wait_disconnected()
        eclient.loop_stop()

        time.sleep(3)
        # session should still exist as we changed the expiry interval on disconnect
        fclient, fcallback = self.new_client(clientid)
        fclient.connect(host="localhost", port=self._test_broker_port, clean_start=False,
                        properties=connect_properties)
        connack = fcallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], True)
        disconnect_properties.SessionExpiryInterval = 0
        fclient.disconnect(properties=disconnect_properties)
        fcallback.wait_disconnected()
        fclient.loop_stop()

        # session should immediately expire
        fclient, fcallback = self.new_client(clientid)
        fclient.connect(host="localhost", port=self._test_broker_port, clean_start=False,
                        properties=connect_properties)
        connack = fcallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        fclient.disconnect()
        fcallback.wait_disconnected()
        fclient.loop_stop()

        fclient.loop_stop()
        eclient.loop_stop()

    def test_user_properties(self):
        clientid = "user properties"
        uclient, ucallback = self.new_client(clientid)
        uclient.loop_start()
        uclient.connect(host="localhost", port=self._test_broker_port)
        ucallback.wait_connected()

        uclient.subscribe(topics[0], qos=2)
        ucallback.wait_subscribed()

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.UserProperty = ("a", "2")
        publish_properties.UserProperty = ("c", "3")
        uclient.publish(topics[0], b"", 0, retain=False,
                        properties=publish_properties)
        uclient.publish(topics[0], b"", 1, retain=False,
                        properties=publish_properties)
        uclient.publish(topics[0], b"", 2, retain=False,
                        properties=publish_properties)
        count = 0
        while len(ucallback.messages) < 3 and count < 50:
            time.sleep(.1)
            count += 1
        uclient.disconnect()
        ucallback.wait_disconnected()
        uclient.loop_stop()
        self.assertEqual(len(ucallback.messages), 3, ucallback.messages)
        userprops = ucallback.messages[0]["message"].properties.UserProperty
        self.assertTrue(userprops in [[("a", "2"), ("c", "3")], [
                        ("c", "3"), ("a", "2")]], userprops)
        userprops = ucallback.messages[1]["message"].properties.UserProperty
        self.assertTrue(userprops in [[("a", "2"), ("c", "3")], [
                        ("c", "3"), ("a", "2")]], userprops)
        userprops = ucallback.messages[2]["message"].properties.UserProperty
        self.assertTrue(userprops in [[("a", "2"), ("c", "3")], [
                        ("c", "3"), ("a", "2")]], userprops)
        qoss = [ucallback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)

    def test_payload_format(self):
        clientid = "payload format"
        pclient, pcallback = self.new_client(clientid)
        pclient.loop_start()
        pclient.connect_async(host="localhost", port=self._test_broker_port)
        response = pcallback.wait_connected()

        pclient.subscribe(topics[0], qos=2)
        response = pcallback.wait_subscribed()
        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.PayloadFormatIndicator = 1
        publish_properties.ContentType = "My name"
        info = pclient.publish(
            topics[0], b"qos 0", 0, retain=False, properties=publish_properties)
        info.wait_for_publish()
        info = pclient.publish(
            topics[0], b"qos 1", 1, retain=False, properties=publish_properties)
        info.wait_for_publish()
        info = pclient.publish(
            topics[0], b"qos 2", 2, retain=False, properties=publish_properties)
        info.wait_for_publish()

        count = 0
        while len(pcallback.messages) < 3 and count < 50:
            time.sleep(.1)
            count += 1
        pclient.disconnect()
        pcallback.wait_disconnected()
        pclient.loop_stop()

        self.assertEqual(len(pcallback.messages), 3, pcallback.messages)
        props = pcallback.messages[0]["message"].properties
        self.assertEqual(props.ContentType, "My name", props.ContentType)
        self.assertEqual(props.PayloadFormatIndicator,
                         1, props.PayloadFormatIndicator)
        props = pcallback.messages[1]["message"].properties
        self.assertEqual(props.ContentType, "My name", props.ContentType)
        self.assertEqual(props.PayloadFormatIndicator,
                         1, props.PayloadFormatIndicator)
        props = pcallback.messages[2]["message"].properties
        self.assertEqual(props.ContentType, "My name", props.ContentType)
        self.assertEqual(props.PayloadFormatIndicator,
                         1, props.PayloadFormatIndicator)
        qoss = [pcallback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)

    def test_message_expiry(self):
        clientid = "message expiry"

        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.SessionExpiryInterval = 99999

        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.loop_start()
        lbclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        response = lbcallback.wait_connected()
        lbclient.subscribe(topics[0], qos=2)
        response = lbcallback.wait_subscribed()
        disconnect_properties = Properties(PacketTypes.DISCONNECT)
        disconnect_properties.SessionExpiryInterval = 999999999
        lbclient.disconnect(properties=disconnect_properties)
        lbcallback.wait_disconnected()
        lbclient.loop_stop()

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.loop_start()
        laclient.connect(host="localhost", port=self._test_broker_port)
        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.MessageExpiryInterval = 1
        laclient.publish(topics[0], b"qos 1 - expire", 1,
                         retain=False, properties=publish_properties)
        laclient.publish(topics[0], b"qos 2 - expire", 2,
                         retain=False, properties=publish_properties)

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.MessageExpiryInterval = 6
        laclient.publish(topics[0], b"qos 1 - don't expire",
                         1, retain=False, properties=publish_properties)
        laclient.publish(topics[0], b"qos 2 - don't expire",
                         2, retain=False, properties=publish_properties)

        time.sleep(3)
        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.loop_start()
        lbclient.connect(host="localhost", port=self._test_broker_port, clean_start=False)
        lbcallback.wait_connected()
        self.waitfor(lbcallback.messages, 1, 3)
        time.sleep(1)
        self.assertEqual(len(lbcallback.messages), 2, lbcallback.messages)
        self.assertTrue(lbcallback.messages[0]["message"].properties.MessageExpiryInterval < 6,
                        lbcallback.messages[0]["message"].properties.MessageExpiryInterval)
        self.assertTrue(lbcallback.messages[1]["message"].properties.MessageExpiryInterval < 6,
                        lbcallback.messages[1]["message"].properties.MessageExpiryInterval)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        lbclient.disconnect()
        lbcallback.wait_disconnected()
        lbclient.loop_stop()

    def test_subscribe_options(self):
        # noLocal
        clientid = 'subscribe options - noLocal'

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        lacallback.wait_connected()
        laclient.loop_start()
        laclient.subscribe(
            topics[0], options=SubscribeOptions(qos=2, noLocal=True))
        lacallback.wait_subscribed()

        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.connect(host="localhost", port=self._test_broker_port)
        lbcallback.wait_connected()
        lbclient.loop_start()
        lbclient.subscribe(
            topics[0], options=SubscribeOptions(qos=2, noLocal=True))
        lbcallback.wait_subscribed()

        laclient.publish(topics[0], b"noLocal test", 1, retain=False)
        self.waitfor(lbcallback.messages, 1, 3)
        time.sleep(1)

        self.assertEqual(lacallback.messages, [], lacallback.messages)
        self.assertEqual(len(lbcallback.messages), 1, lbcallback.messages)
        laclient.disconnect()
        lacallback.wait_disconnected()
        lbclient.disconnect()
        lbcallback.wait_disconnected()
        laclient.loop_stop()
        lbclient.loop_stop()

        # retainAsPublished
        clientid = 'subscribe options - retain as published'
        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        lacallback.wait_connected()
        laclient.subscribe(topics[0], options=SubscribeOptions(
            qos=2, retainAsPublished=True))
        lacallback.wait_subscribed()
        self.waitfor(lacallback.subscribeds, 1, 3)
        laclient.publish(
            topics[0], b"retain as published false", 1, retain=False)
        laclient.publish(
            topics[0], b"retain as published true", 1, retain=True)

        self.waitfor(lacallback.messages, 2, 3)
        time.sleep(1)

        self.assertEqual(len(lacallback.messages), 2, lacallback.messages)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()
        self.assertEqual(lacallback.messages[0]["message"].retain, False)
        self.assertEqual(lacallback.messages[1]["message"].retain, True)

        # retainHandling
        clientid = 'subscribe options - retain handling'
        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        lacallback.wait_connected()
        laclient.publish(topics[1], b"qos 0", 0, retain=True)
        laclient.publish(topics[2], b"qos 1", 1, retain=True)
        laclient.publish(topics[3], b"qos 2", 2, retain=True)
        time.sleep(1)

        # retain handling 1 only gives us retained messages on a new subscription
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=1))
        lacallback.wait_subscribed()
        self.assertEqual(len(lacallback.messages), 3)
        qoss = [lacallback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)
        lacallback.clear()
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=1))
        lacallback.wait_subscribed()
        time.sleep(1)
        self.assertEqual(len(lacallback.messages), 0)

        # remove that subscription
        properties = Properties(PacketTypes.UNSUBSCRIBE)
        properties.UserProperty = ("a", "2")
        properties.UserProperty = ("c", "3")
        laclient.unsubscribe(wildtopics[5], properties)
        response = lacallback.wait_unsubscribed()

        # check that we really did remove that subscription
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=1))
        lacallback.wait_subscribed()
        self.assertEqual(len(lacallback.messages), 3)
        qoss = [lacallback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)
        lacallback.clear()
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=1))
        lacallback.wait_subscribed()
        time.sleep(1)
        self.assertEqual(len(lacallback.messages), 0)

        # remove that subscription
        properties = Properties(PacketTypes.UNSUBSCRIBE)
        properties.UserProperty = ("a", "2")
        properties.UserProperty = ("c", "3")
        laclient.unsubscribe(wildtopics[5], properties)
        response = lacallback.wait_unsubscribed()

        lacallback.clear()
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=2))
        lacallback.wait_subscribed()
        self.assertEqual(len(lacallback.messages), 0)
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=2))
        lacallback.wait_subscribed()
        self.assertEqual(len(lacallback.messages), 0)

        # remove that subscription
        laclient.unsubscribe(wildtopics[5])
        response = lacallback.wait_unsubscribed()

        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=0))
        lacallback.wait_subscribed()
        self.assertEqual(len(lacallback.messages), 3)
        qoss = [lacallback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)
        lacallback.clear()
        laclient.subscribe(
            wildtopics[5], options=SubscribeOptions(2, retainHandling=0))
        time.sleep(1)
        self.assertEqual(len(lacallback.messages), 3)
        qoss = [lacallback.messages[i]["message"].qos for i in range(3)]
        self.assertTrue(1 in qoss and 2 in qoss and 0 in qoss, qoss)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        cleanRetained(self._test_broker_port)

    def test_subscription_identifiers(self):
        clientid = 'subscription identifiers'

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        lacallback.wait_connected()
        laclient.loop_start()

        sub_properties = Properties(PacketTypes.SUBSCRIBE)
        sub_properties.SubscriptionIdentifier = 456789
        laclient.subscribe(topics[0], qos=2, properties=sub_properties)
        lacallback.wait_subscribed()

        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.connect(host="localhost", port=self._test_broker_port)
        lbcallback.wait_connected()
        lbclient.loop_start()
        sub_properties = Properties(PacketTypes.SUBSCRIBE)
        sub_properties.SubscriptionIdentifier = 2
        lbclient.subscribe(topics[0], qos=2, properties=sub_properties)
        lbcallback.wait_subscribed()

        sub_properties.clear()
        sub_properties.SubscriptionIdentifier = 3
        lbclient.subscribe(topics[0]+"/#", qos=2, properties=sub_properties)

        lbclient.publish(topics[0], b"sub identifier test", 1, retain=False)

        self.waitfor(lacallback.messages, 1, 3)
        self.assertEqual(len(lacallback.messages), 1, lacallback.messages)
        self.assertEqual(lacallback.messages[0]["message"].properties.SubscriptionIdentifier[0],
                         456789, lacallback.messages[0]["message"].properties.SubscriptionIdentifier)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        self.waitfor(lbcallback.messages, 1, 3)
        self.assertEqual(len(lbcallback.messages), 1, lbcallback.messages)
        expected_subsids = set([2, 3])
        received_subsids = set(
            lbcallback.messages[0]["message"].properties.SubscriptionIdentifier)
        self.assertEqual(received_subsids, expected_subsids, received_subsids)
        lbclient.disconnect()
        lbcallback.wait_disconnected()
        lbclient.loop_stop()

    def test_request_response(self):
        clientid = 'request response'

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        lacallback.wait_connected()
        laclient.loop_start()

        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.connect(host="localhost", port=self._test_broker_port)
        lbcallback.wait_connected()
        lbclient.loop_start()

        laclient.subscribe(
            topics[0], options=SubscribeOptions(2, noLocal=True))
        lacallback.wait_subscribed()

        lbclient.subscribe(
            topics[0], options=SubscribeOptions(2, noLocal=True))
        lbcallback.wait_subscribed()

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.ResponseTopic = topics[0]
        publish_properties.CorrelationData = b"334"
        # client a is the requester
        laclient.publish(topics[0], b"request", 1,
                         properties=publish_properties)

        # client b is the responder
        self.waitfor(lbcallback.messages, 1, 3)
        self.assertEqual(len(lbcallback.messages), 1, lbcallback.messages)
        self.assertEqual(lbcallback.messages[0]["message"].properties.ResponseTopic, topics[0],
                         lbcallback.messages[0]["message"].properties)
        self.assertEqual(lbcallback.messages[0]["message"].properties.CorrelationData, b"334",
                         lbcallback.messages[0]["message"].properties)

        lbclient.publish(lbcallback.messages[0]["message"].properties.ResponseTopic, b"response", 1,
                         properties=lbcallback.messages[0]["message"].properties)

        # client a gets the response
        self.waitfor(lacallback.messages, 1, 3)
        self.assertEqual(len(lacallback.messages), 1, lacallback.messages)

        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()
        lbclient.disconnect()
        lbcallback.wait_disconnected()
        lbclient.loop_stop()

    def test_client_topic_alias(self):
        clientid = 'client topic alias'

        # no server side topic aliases allowed
        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.TopicAlias = 0  # topic alias 0 not allowed
        laclient.publish(topics[0], "topic alias 0", 1,
                         properties=publish_properties)

        # should get back a disconnect with Topic alias invalid
        lacallback.wait_disconnected()
        laclient.loop_stop()

        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.TopicAliasMaximum = 0  # server topic aliases not allowed
        connect_properties.SessionExpiryInterval = 99999
        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lacallback.wait_connected()
        clientTopicAliasMaximum = 0
        if hasattr(connack["properties"], "TopicAliasMaximum"):
            clientTopicAliasMaximum = connack["properties"].TopicAliasMaximum

        if clientTopicAliasMaximum == 0:
            laclient.disconnect()
            lacallback.wait_disconnected()
            laclient.loop_stop()
            return

        laclient.subscribe(topics[0], qos=2)
        lacallback.wait_subscribed()

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.TopicAlias = 1
        laclient.publish(topics[0], b"topic alias 1",
                         1, properties=publish_properties)
        self.waitfor(lacallback.messages, 1, 3)
        self.assertEqual(len(lacallback.messages), 1, lacallback.messages)

        laclient.publish("", b"topic alias 2", 1,
                         properties=publish_properties)
        self.waitfor(lacallback.messages, 2, 3)
        self.assertEqual(len(lacallback.messages), 2, lacallback.messages)

        laclient.disconnect()  # should get rid of the topic aliases but not subscriptions
        lacallback.wait_disconnected()
        laclient.loop_stop()

        # check aliases have been deleted
        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port, clean_start=False,
                         properties=connect_properties)

        laclient.publish(topics[0], b"topic alias 3", 1)
        self.waitfor(lacallback.messages, 1, 3)
        self.assertEqual(len(lacallback.messages), 1, lacallback.messages)

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.TopicAlias = 1
        laclient.publish("", b"topic alias 4", 1,
                         properties=publish_properties)

        # should get back a disconnect with Topic alias invalid
        lacallback.wait_disconnected()
        laclient.loop_stop()

    def test_server_topic_alias(self):
        clientid = 'server topic alias'

        serverTopicAliasMaximum = 1  # server topic alias allowed
        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.TopicAliasMaximum = serverTopicAliasMaximum

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lacallback.wait_connected()
        laclient.loop_start()
        clientTopicAliasMaximum = 0
        if hasattr(connack["properties"], "TopicAliasMaximum"):
            clientTopicAliasMaximum = connack["properties"].TopicAliasMaximum

        laclient.subscribe(topics[0], qos=2)
        lacallback.wait_subscribed()

        for qos in range(3):
            laclient.publish(topics[0], b"topic alias 1", qos)
        self.waitfor(lacallback.messages, 3, 3)
        self.assertEqual(len(lacallback.messages), 3, lacallback.messages)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        # first message should set the topic alias
        self.assertTrue(hasattr(
            lacallback.messages[0]["message"].properties, "TopicAlias"), lacallback.messages[0]["message"].properties)
        topicalias = lacallback.messages[0]["message"].properties.TopicAlias

        self.assertTrue(topicalias > 0)
        self.assertEqual(lacallback.messages[0]["message"].topic, topics[0])

        self.assertEqual(
            lacallback.messages[1]["message"].properties.TopicAlias, topicalias)
        self.assertEqual(lacallback.messages[1]["message"].topic, "")

        self.assertEqual(
            lacallback.messages[2]["message"].properties.TopicAlias, topicalias)
        self.assertEqual(lacallback.messages[2]["message"].topic, "")

        serverTopicAliasMaximum = 0  # no server topic alias allowed
        connect_properties = Properties(PacketTypes.CONNECT)
        # connect_properties.TopicAliasMaximum = serverTopicAliasMaximum # default is 0

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        clientTopicAliasMaximum = 0
        if hasattr(connack["properties"], "TopicAliasMaximum"):
            clientTopicAliasMaximum = connack["properties"].TopicAliasMaximum

        laclient.subscribe(topics[0], qos=2)
        lacallback.wait_subscribed()

        for qos in range(3):
            laclient.publish(topics[0], b"topic alias 2", qos)
        self.waitfor(lacallback.messages, 3, 3)
        self.assertEqual(len(lacallback.messages), 3, lacallback.messages)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        # No topic aliases
        self.assertFalse(hasattr(
            lacallback.messages[0]["message"].properties, "TopicAlias"), lacallback.messages[0]["message"].properties)
        self.assertFalse(hasattr(
            lacallback.messages[1]["message"].properties, "TopicAlias"), lacallback.messages[1]["message"].properties)
        self.assertFalse(hasattr(
            lacallback.messages[2]["message"].properties, "TopicAlias"), lacallback.messages[2]["message"].properties)

        serverTopicAliasMaximum = 0  # no server topic alias allowed
        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.TopicAliasMaximum = serverTopicAliasMaximum  # default is 0

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        clientTopicAliasMaximum = 0
        if hasattr(connack["properties"], "TopicAliasMaximum"):
            clientTopicAliasMaximum = connack["properties"].TopicAliasMaximum

        laclient.subscribe(topics[0], qos=2)
        lacallback.wait_subscribed()

        for qos in range(3):
            laclient.publish(topics[0], b"topic alias 3", qos)
        self.waitfor(lacallback.messages, 3, 3)
        self.assertEqual(len(lacallback.messages), 3, lacallback.messages)
        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        # No topic aliases
        self.assertFalse(hasattr(
            lacallback.messages[0]["message"].properties, "TopicAlias"), lacallback.messages[0]["message"].properties)
        self.assertFalse(hasattr(
            lacallback.messages[1]["message"].properties, "TopicAlias"), lacallback.messages[1]["message"].properties)
        self.assertFalse(hasattr(
            lacallback.messages[2]["message"].properties, "TopicAlias"), lacallback.messages[2]["message"].properties)

    def test_maximum_packet_size(self):
        clientid = 'maximum packet size'

        # 1. server max packet size
        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        serverMaximumPacketSize = 2**28-1
        if hasattr(connack["properties"], "MaximumPacketSize"):
            serverMaximumPacketSize = connack["properties"].MaximumPacketSize

        if serverMaximumPacketSize < 65535:
            # publish bigger packet than server can accept
            payload = b"."*serverMaximumPacketSize
            laclient.publish(topics[0], payload, 0)
            # should get back a disconnect with packet size too big
            response = lacallback.wait_disconnected()
            self.assertEqual(len(lacallback.disconnecteds),
                             0, lacallback.disconnecteds)
            self.assertEqual(response["reasonCode"].getName(),
                             "Packet too large", response["reasonCode"].getName())
        else:
            laclient.disconnect()
            lacallback.wait_disconnected()
        laclient.loop_stop()

        # 1. client max packet size
        maximumPacketSize = 64  # max packet size we want to receive
        connect_properties = Properties(PacketTypes.CONNECT)
        connect_properties.MaximumPacketSize = maximumPacketSize

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        serverMaximumPacketSize = 2**28-1
        if hasattr(connack["properties"], "MaximumPacketSize"):
            serverMaximumPacketSize = connack["properties"].MaximumPacketSize

        laclient.subscribe(topics[0], qos=2)
        response = lacallback.wait_subscribed()

        # send a small enough packet, should get this one back
        payload = b"."*(int(maximumPacketSize/2))
        laclient.publish(topics[0], payload, 0)
        self.waitfor(lacallback.messages, 1, 3)
        self.assertEqual(len(lacallback.messages), 1, lacallback.messages)

        # send a packet too big to receive
        payload = b"."*maximumPacketSize
        laclient.publish(topics[0], payload, 1)
        self.waitfor(lacallback.messages, 2, 3)
        self.assertEqual(len(lacallback.messages), 1, lacallback.messages)

        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

    """
    def test_server_keep_alive(self):
        clientid = 'server keep alive'

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        self.assertTrue(hasattr(connack["properties"], "ServerKeepAlive"))
        self.assertEqual(connack["properties"].ServerKeepAlive, 60)

        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()
    """

    def test_will_delay(self):
        # the will message should be received earlier than the session expiry

        clientid = 'will delay'

        will_properties = Properties(PacketTypes.WILLMESSAGE)
        connect_properties = Properties(PacketTypes.CONNECT)

        # set the will delay and session expiry to the same value -
        # then both should occur at the same time
        will_properties.WillDelayInterval = 3  # in seconds
        connect_properties.SessionExpiryInterval = 5

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.will_set(
            topics[0], payload=b"test_will_delay will message", properties=will_properties)
        laclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lacallback.wait_connected()
        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)
        laclient.loop_start()

        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.connect(host="localhost", port=self._test_broker_port, properties=connect_properties)
        connack = lbcallback.wait_connected()
        lbclient.loop_start()
        # subscribe to will message topic
        lbclient.subscribe(topics[0], qos=2)
        lbcallback.wait_subscribed()

        # abort client a and wait for the will message
        laclient.loop_stop()
        laclient.socket().close()
        start = time.time()
        while lbcallback.messages == []:
            time.sleep(.1)
        duration = time.time() - start
        self.assertAlmostEqual(duration, 4, delta=1)
        self.assertEqual(lbcallback.messages[0]["message"].topic, topics[0])
        self.assertEqual(
            lbcallback.messages[0]["message"].payload, b"test_will_delay will message")

        lbclient.disconnect()
        lbcallback.wait_disconnected()
        lbclient.loop_stop()

    def test_shared_subscriptions(self):
        clientid = 'shared subscriptions'

        shared_sub_topic = '$share/sharename/' + topic_prefix + 'x'
        shared_pub_topic = topic_prefix + 'x'

        laclient, lacallback = self.new_client(clientid+" a")
        laclient.connect(host="localhost", port=self._test_broker_port)
        connack = lacallback.wait_connected()
        laclient.loop_start()

        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)

        laclient.subscribe(
            [(shared_sub_topic, SubscribeOptions(2)), (topics[0], SubscribeOptions(2))])
        response = lacallback.wait_subscribed()

        lbclient, lbcallback = self.new_client(clientid+" b")
        lbclient.connect(host="localhost", port=self._test_broker_port)
        connack = lbcallback.wait_connected()
        lbclient.loop_start()

        self.assertEqual(connack["reasonCode"].getName(), "Success")
        self.assertEqual(connack["flags"]["session present"], False)

        lbclient.subscribe(
            [(shared_sub_topic, SubscribeOptions(2)), (topics[0], 2)])
        response = lbcallback.wait_subscribed()

        lacallback.clear()
        lbcallback.clear()

        count = 1
        for i in range(count):
            lbclient.publish(topics[0], "message "+str(i), 0)
        j = 0
        while len(lacallback.messages) + len(lbcallback.messages) < 2*count and j < 20:
            time.sleep(.1)
            j += 1
        time.sleep(1)
        self.assertEqual(len(lacallback.messages), count)
        self.assertEqual(len(lbcallback.messages), count)

        lacallback.clear()
        lbcallback.clear()

        for i in range(count):
            lbclient.publish(shared_pub_topic, "message "+str(i), 0)
        j = 0
        while len(lacallback.messages) + len(lbcallback.messages) < count and j < 20:
            time.sleep(.1)
            j += 1
        time.sleep(1)
        # Each message should only be received once
        self.assertEqual(len(lacallback.messages) +
                         len(lbcallback.messages), count)

        laclient.disconnect()
        lacallback.wait_disconnected()
        laclient.loop_stop()

        lbclient.disconnect()
        lbcallback.wait_disconnected()
        lbclient.loop_stop()


def setData():
    global topics, wildtopics, nosubscribe_topics, topic_prefix
    topics = ("TopicA", "TopicA/B", "Topic/C", "TopicA/C", "/TopicA")
    wildtopics = ("TopicA/+", "+/C", "#", "/#", "/+", "+/+", "TopicA/#")
    nosubscribe_topics = ("test/nosubscribe",)
    topic_prefix = "paho.mqtt.client.mqttv5/"



"""
*******************************************************************
  Copyright (c) 2017, 2019 IBM Corp.

  All rights reserved. This program and the accompanying materials
  are made available under the terms of the Eclipse Public License v1.0
  and Eclipse Distribution License v1.0 which accompany this distribution.

  The Eclipse Public License is available at
     http://www.eclipse.org/legal/epl-v10.html
  and the Eclipse Distribution License is available at
    http://www.eclipse.org/org/documents/edl-v10.php.

  Contributors:
     Ian Craggs - initial implementation and/or documentation
*******************************************************************
"""


class PacketTypes:

    """
    Packet types class.  Includes the AUTH packet for MQTT v5.0.

    Holds constants for each packet type such as PacketTypes.PUBLISH
    and packet name strings: PacketTypes.Names[PacketTypes.PUBLISH].

    """

    indexes = range(1, 16)

    # Packet types
    CONNECT, CONNACK, PUBLISH, PUBACK, PUBREC, PUBREL, \
        PUBCOMP, SUBSCRIBE, SUBACK, UNSUBSCRIBE, UNSUBACK, \
        PINGREQ, PINGRESP, DISCONNECT, AUTH = indexes

    # Dummy packet type for properties use - will delay only applies to will
    WILLMESSAGE = 99

    Names = [ "reserved", \
    "Connect", "Connack", "Publish", "Puback", "Pubrec", "Pubrel", \
    "Pubcomp", "Subscribe", "Suback", "Unsubscribe", "Unsuback", \
    "Pingreq", "Pingresp", "Disconnect", "Auth"]



#!/usr/bin/env python

# Test whether a client sends a pingreq after the keepalive time

# The client should connect to port 1888 with keepalive=4, clean session set,
# and client id 01-keepalive-pingreq
# The client should send a PINGREQ message after the appropriate amount of time
# (4 seconds after no traffic).

import time

import context
import paho_test

rc = 1
keepalive = 4
connect_packet = paho_test.gen_connect("01-keepalive-pingreq", keepalive=keepalive)
connack_packet = paho_test.gen_connack(rc=0)

pingreq_packet = paho_test.gen_pingreq()
pingresp_packet = paho_test.gen_pingresp()

sock = paho_test.create_server_socket()

client = context.start_client()

try:
    (conn, address) = sock.accept()
    conn.settimeout(keepalive+10)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        conn.send(connack_packet)

        if paho_test.expect_packet(conn, "pingreq", pingreq_packet):
            time.sleep(1.0)
            conn.send(pingresp_packet)

            if paho_test.expect_packet(conn, "pingreq", pingreq_packet):
                rc = 0

    conn.close()
finally:
    client.terminate()
    client.wait()
    sock.close()

exit(rc)




import os
import sys
import time
import inspect
import unicodedata

import pytest
import paho.mqtt.client as client

# From http://stackoverflow.com/questions/279237/python-import-a-module-from-a-folder
cmd_subfolder = os.path.realpath(
    os.path.abspath(
        os.path.join(
            os.path.split(
                inspect.getfile(inspect.currentframe()))[0],
            '..', 'test')))
if cmd_subfolder not in sys.path:
    sys.path.insert(0, cmd_subfolder)
import paho_test

# Import test fixture
from testsupport.broker import fake_broker


@pytest.mark.parametrize("proto_ver", [
    (client.MQTTv31),
    (client.MQTTv311),
])
class Test_connect(object):
    """
    Tests on connect/disconnect behaviour of the client
    """

    def test_01_con_discon_success(self, proto_ver, fake_broker):
        mqttc = client.Client(
            "01-con-discon-success", protocol=proto_ver)

        def on_connect(mqttc, obj, flags, rc):
            assert rc == 0
            mqttc.disconnect()

        mqttc.on_connect = on_connect

        mqttc.connect_async("localhost", 1888)
        mqttc.loop_start()

        try:
            fake_broker.start()

            connect_packet = paho_test.gen_connect(
                "01-con-discon-success", keepalive=60,
                proto_ver=proto_ver)
            packet_in = fake_broker.receive_packet(1000)
            assert packet_in  # Check connection was not closed
            assert packet_in == connect_packet

            connack_packet = paho_test.gen_connack(rc=0)
            count = fake_broker.send_packet(connack_packet)
            assert count  # Check connection was not closed
            assert count == len(connack_packet)

            disconnect_packet = paho_test.gen_disconnect()
            packet_in = fake_broker.receive_packet(1000)
            assert packet_in  # Check connection was not closed
            assert packet_in == disconnect_packet

        finally:
            mqttc.loop_stop()

        packet_in = fake_broker.receive_packet(1)
        assert not packet_in  # Check connection is closed

    def test_01_con_failure_rc(self, proto_ver, fake_broker):
        mqttc = client.Client(
            "01-con-failure-rc", protocol=proto_ver)

        def on_connect(mqttc, obj, flags, rc):
            assert rc == 1

        mqttc.on_connect = on_connect

        mqttc.connect_async("localhost", 1888)
        mqttc.loop_start()

        try:
            fake_broker.start()

            connect_packet = paho_test.gen_connect(
                "01-con-failure-rc", keepalive=60,
                proto_ver=proto_ver)
            packet_in = fake_broker.receive_packet(1000)
            assert packet_in  # Check connection was not closed
            assert packet_in == connect_packet

            connack_packet = paho_test.gen_connack(rc=1)
            count = fake_broker.send_packet(connack_packet)
            assert count  # Check connection was not closed
            assert count == len(connack_packet)

            packet_in = fake_broker.receive_packet(1)
            assert not packet_in  # Check connection is closed

        finally:
            mqttc.loop_stop()


class TestPublishBroker2Client(object):

    def test_invalid_utf8_topic(self, fake_broker):
        mqttc = client.Client("client-id")

        def on_message(client, userdata, msg):
            with pytest.raises(UnicodeDecodeError):
                msg.topic
            client.disconnect()

        mqttc.on_message = on_message

        mqttc.connect_async("localhost", 1888)
        mqttc.loop_start()

        try:
            fake_broker.start()

            connect_packet = paho_test.gen_connect("client-id")
            packet_in = fake_broker.receive_packet(len(connect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == connect_packet

            connack_packet = paho_test.gen_connack(rc=0)
            count = fake_broker.send_packet(connack_packet)
            assert count  # Check connection was not closed
            assert count == len(connack_packet)

            publish_packet = paho_test.gen_publish(b"\xff", qos=0)
            count = fake_broker.send_packet(publish_packet)
            assert count  # Check connection was not closed
            assert count == len(publish_packet)

            disconnect_packet = paho_test.gen_disconnect()
            packet_in = fake_broker.receive_packet(len(disconnect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == disconnect_packet

        finally:
            mqttc.loop_stop()

        packet_in = fake_broker.receive_packet(1)
        assert not packet_in  # Check connection is closed

    def test_valid_utf8_topic_recv(self, fake_broker):
        mqttc = client.Client("client-id")

        # It should be non-ascii multi-bytes character
        topic = unicodedata.lookup('SNOWMAN')

        def on_message(client, userdata, msg):
            assert msg.topic == topic
            client.disconnect()

        mqttc.on_message = on_message

        mqttc.connect_async("localhost", 1888)
        mqttc.loop_start()

        try:
            fake_broker.start()

            connect_packet = paho_test.gen_connect("client-id")
            packet_in = fake_broker.receive_packet(len(connect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == connect_packet

            connack_packet = paho_test.gen_connack(rc=0)
            count = fake_broker.send_packet(connack_packet)
            assert count  # Check connection was not closed
            assert count == len(connack_packet)

            publish_packet = paho_test.gen_publish(
                topic.encode('utf-8'), qos=0
            )
            count = fake_broker.send_packet(publish_packet)
            assert count  # Check connection was not closed
            assert count == len(publish_packet)

            disconnect_packet = paho_test.gen_disconnect()
            packet_in = fake_broker.receive_packet(len(disconnect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == disconnect_packet

        finally:
            mqttc.loop_stop()

        packet_in = fake_broker.receive_packet(1)
        assert not packet_in  # Check connection is closed

    def test_valid_utf8_topic_publish(self, fake_broker):
        mqttc = client.Client("client-id")

        # It should be non-ascii multi-bytes character
        topic = unicodedata.lookup('SNOWMAN')

        mqttc.connect_async("localhost", 1888)
        mqttc.loop_start()

        try:
            fake_broker.start()

            connect_packet = paho_test.gen_connect("client-id")
            packet_in = fake_broker.receive_packet(len(connect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == connect_packet

            connack_packet = paho_test.gen_connack(rc=0)
            count = fake_broker.send_packet(connack_packet)
            assert count  # Check connection was not closed
            assert count == len(connack_packet)

            mqttc.publish(topic, None, 0)
            # Small sleep needed to avoid connection reset.
            time.sleep(0.3)

            publish_packet = paho_test.gen_publish(
                topic.encode('utf-8'), qos=0
            )
            packet_in = fake_broker.receive_packet(len(publish_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == publish_packet

            mqttc.disconnect()

            disconnect_packet = paho_test.gen_disconnect()
            packet_in = fake_broker.receive_packet(len(disconnect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == disconnect_packet

        finally:
            mqttc.loop_stop()

        packet_in = fake_broker.receive_packet(1)
        assert not packet_in  # Check connection is closed

    def test_message_callback(self, fake_broker):
        mqttc = client.Client("client-id")
        userdata = {
            'on_message': 0,
            'callback1': 0,
            'callback2': 0,
        }
        mqttc.user_data_set(userdata)

        def on_message(client, userdata, msg):
            assert msg.topic == 'topic/value'
            userdata['on_message'] += 1

        def callback1(client, userdata, msg):
            assert msg.topic == 'topic/callback/1'
            userdata['callback1'] += 1

        def callback2(client, userdata, msg):
            assert msg.topic in ('topic/callback/3', 'topic/callback/1')
            userdata['callback2'] += 1

        mqttc.on_message = on_message
        mqttc.message_callback_add('topic/callback/1', callback1)
        mqttc.message_callback_add('topic/callback/+', callback2)

        mqttc.connect_async("localhost", 1888)
        mqttc.loop_start()

        try:
            fake_broker.start()

            connect_packet = paho_test.gen_connect("client-id")
            packet_in = fake_broker.receive_packet(len(connect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == connect_packet

            connack_packet = paho_test.gen_connack(rc=0)
            count = fake_broker.send_packet(connack_packet)
            assert count  # Check connection was not closed
            assert count == len(connack_packet)

            publish_packet = paho_test.gen_publish(b"topic/value", qos=1, mid=1)
            count = fake_broker.send_packet(publish_packet)
            assert count  # Check connection was not closed
            assert count == len(publish_packet)

            publish_packet = paho_test.gen_publish(b"topic/callback/1", qos=1, mid=2)
            count = fake_broker.send_packet(publish_packet)
            assert count  # Check connection was not closed
            assert count == len(publish_packet)

            publish_packet = paho_test.gen_publish(b"topic/callback/3", qos=1, mid=3)
            count = fake_broker.send_packet(publish_packet)
            assert count  # Check connection was not closed
            assert count == len(publish_packet)


            puback_packet = paho_test.gen_puback(mid=1)
            packet_in = fake_broker.receive_packet(len(puback_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == puback_packet

            puback_packet = paho_test.gen_puback(mid=2)
            packet_in = fake_broker.receive_packet(len(puback_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == puback_packet

            puback_packet = paho_test.gen_puback(mid=3)
            packet_in = fake_broker.receive_packet(len(puback_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == puback_packet

            mqttc.disconnect()

            disconnect_packet = paho_test.gen_disconnect()
            packet_in = fake_broker.receive_packet(len(disconnect_packet))
            assert packet_in  # Check connection was not closed
            assert packet_in == disconnect_packet

        finally:
            mqttc.loop_stop()

        packet_in = fake_broker.receive_packet(1)
        assert not packet_in  # Check connection is closed

        assert userdata['on_message'] == 1
        assert userdata['callback1'] == 1
        assert userdata['callback2'] == 2



import socket
import sys

if sys.version_info < (3, 0):
    from mock import Mock
else:
    from unittest.mock import Mock

import pytest
from paho.mqtt.client import WebsocketWrapper, WebsocketConnectionError


class TestHeaders(object):
    """ Make sure headers are used correctly """

    def test_normal_headers(self):
        """ Normal headers as specified in RFC 6455 """

        response = [
            "HTTP/1.1 101 Switching Protocols",
            "Upgrade: websocket",
            "Connection: Upgrade",
            "Sec-WebSocket-Accept: badreturnvalue=",
            "Sec-WebSocket-Protocol: chat",
            "\r\n",
        ]

        def iter_response():
            for i in "\r\n".join(response).encode("utf8"):
                yield i

            for i in "\r\n".encode("utf8"):
                yield i

        it = iter_response()

        def fakerecv(*args):
            if sys.version_info < (3, 0):
                return next(it)
            else:
                return bytes([next(it)])

        mocksock = Mock(
            spec_set=socket.socket,
            recv=fakerecv,
            send=Mock(),
        )

        wargs = {
            "host": "testhost.com",
            "port": 1234,
            "path": "/mqtt",
            "extra_headers": None,
            "is_ssl": True,
            "socket": mocksock,
        }

        with pytest.raises(WebsocketConnectionError) as exc:
            WebsocketWrapper(**wargs)

        # We're not creating the response hash properly so it should raise this
        # error
        assert str(exc.value) == "WebSocket handshake error, invalid secret key"

        expected_sent = [i.format(**wargs) for i in [
            "GET {path:s} HTTP/1.1",
            "Host: {host:s}",
            "Upgrade: websocket",
            "Connection: Upgrade",
            "Sec-Websocket-Protocol: mqtt",
            "Sec-Websocket-Version: 13",
            "Origin: https://{host:s}:{port:d}",
        ]]

        # Only sends the header once
        assert mocksock.send.call_count == 1

        for i in expected_sent:
            assert i in mocksock.send.call_args[0][0].decode("utf8")



#!/usr/bin/env python3

import socket
import uuid
import paho.mqtt.client as mqtt
from select import select
from time import time

client_id = 'paho-mqtt-python/issue72/' + str(uuid.uuid4())
topic = client_id
print("Using client_id / topic: " + client_id)


class SelectMqttExample:
    def __init__(self):
        pass

    def on_connect(self, client, userdata, flags, rc):
        print("Subscribing")
        client.subscribe(topic)

    def on_message(self, client, userdata, msg):
        if self.state not in {1, 3, 5}:
            print("Got unexpected message: {}".format(msg.decode()))
            return

        print("Got message with len {}".format(len(msg.payload)))
        self.state += 1
        self.t = time()

    def on_disconnect(self, client, userdata, rc):
        self.disconnected = True, rc

    def do_select(self):
        sock = self.client.socket()
        if not sock:
            raise Exception("Socket is gone")

        print("Selecting for reading" + (" and writing" if self.client.want_write() else ""))
        r, w, e = select(
            [sock],
            [sock] if self.client.want_write() else [],
            [],
            1
        )

        if sock in r:
            print("Socket is readable, calling loop_read")
            self.client.loop_read()

        if sock in w:
            print("Socket is writable, calling loop_write")
            self.client.loop_write()

        self.client.loop_misc()

    def main(self):
        self.disconnected = (False, None)
        self.t = time()
        self.state = 0

        self.client = mqtt.Client(client_id=client_id)
        self.client.on_connect = self.on_connect
        self.client.on_message = self.on_message
        self.client.on_disconnect = self.on_disconnect

        self.client.connect('mqtt.eclipse.org', 1883, 60)
        print("Socket opened")
        self.client.socket().setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)

        while not self.disconnected[0]:
            self.do_select()

            if self.state in {0, 2, 4}:
                if time() - self.t >= 5:
                    print("Publishing")
                    self.client.publish(topic, b'Hello' * 40000)
                    self.state += 1

            if self.state == 6:
                self.state += 1
                self.client.disconnect()

        print("Disconnected: {}".format(self.disconnected[1]))


print("Starting")
SelectMqttExample().main()
print("Finished")



import pytest

import paho.mqtt.client as client


class Test_client_function(object):
    """
    Tests on topic_matches_sub function in the client module
    """

    @pytest.mark.parametrize("sub,topic", [
        ("foo/bar", "foo/bar"),
        ("foo/+", "foo/bar"),
        ("foo/+/baz", "foo/bar/baz"),
        ("foo/+/#", "foo/bar/baz"),
        ("A/B/+/#", "A/B/B/C"),
        ("#", "foo/bar/baz"),
        ("#", "/foo/bar"),
        ("/#", "/foo/bar"),
        ("$SYS/bar", "$SYS/bar"),
    ])
    def test_matching(self, sub, topic):
        assert client.topic_matches_sub(sub, topic)


    @pytest.mark.parametrize("sub,topic", [
        ("test/6/#", "test/3"),
        ("foo/bar", "foo"),
        ("foo/+", "foo/bar/baz"),
        ("foo/+/baz", "foo/bar/bar"),
        ("foo/+/#", "fo2/bar/baz"),
        ("/#", "foo/bar"),
        ("#", "$SYS/bar"),
        ("$BOB/bar", "$SYS/bar"),
    ])
    def test_not_matching(self, sub, topic):
        assert not client.topic_matches_sub(sub, topic)



#!/usr/bin/env python

# Test whether a client sends a correct PUBLISH to a topic with QoS 2 and responds to a disconnect.

import context
import paho_test

rc = 1
keepalive = 60
connect_packet = paho_test.gen_connect(
    "publish-qos2-test", keepalive=keepalive, clean_session=False,
)
connack_packet = paho_test.gen_connack(rc=0)

disconnect_packet = paho_test.gen_disconnect()

mid = 1
publish_packet = paho_test.gen_publish(
    u"pub/qos2/test", qos=2, mid=mid, payload="message".encode('utf-8'))
publish_dup_packet = paho_test.gen_publish(
    u"pub/qos2/test", qos=2, mid=mid, payload="message".encode('utf-8'), dup=True)
pubrec_packet = paho_test.gen_pubrec(mid)
pubrel_packet = paho_test.gen_pubrel(mid)
pubcomp_packet = paho_test.gen_pubcomp(mid)

sock = paho_test.create_server_socket()

client = context.start_client()

try:
    (conn, address) = sock.accept()
    conn.settimeout(5)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        conn.send(connack_packet)

        if paho_test.expect_packet(conn, "publish", publish_packet):
            # Disconnect client. It should reconnect.
            conn.close()

            (conn, address) = sock.accept()
            conn.settimeout(15)

            if paho_test.expect_packet(conn, "connect", connect_packet):
                conn.send(connack_packet)

                if paho_test.expect_packet(conn, "retried publish", publish_dup_packet):
                    conn.send(pubrec_packet)

                    if paho_test.expect_packet(conn, "pubrel", pubrel_packet):
                        # Disconnect client. It should reconnect.
                        conn.close()

                        (conn, address) = sock.accept()
                        conn.settimeout(15)

                        # Complete connection and message flow.
                        if paho_test.expect_packet(conn, "connect", connect_packet):
                            conn.send(connack_packet)

                            if paho_test.expect_packet(conn, "retried pubrel", pubrel_packet):
                                conn.send(pubcomp_packet)

                                if paho_test.expect_packet(conn, "disconnect", disconnect_packet):
                                    rc = 0

    conn.close()
finally:
    client.terminate()
    client.wait()
    sock.close()

exit(rc)



"""
*******************************************************************
  Copyright (c) 2017, 2019 IBM Corp.

  All rights reserved. This program and the accompanying materials
  are made available under the terms of the Eclipse Public License v1.0
  and Eclipse Distribution License v1.0 which accompany this distribution.

  The Eclipse Public License is available at
     http://www.eclipse.org/legal/epl-v10.html
  and the Eclipse Distribution License is available at
    http://www.eclipse.org/org/documents/edl-v10.php.

  Contributors:
     Ian Craggs - initial implementation and/or documentation
*******************************************************************
"""

import sys, struct

from .packettypes import PacketTypes


class MQTTException(Exception):
    pass


class MalformedPacket(MQTTException):
    pass


def writeInt16(length):
    # serialize a 16 bit integer to network format
    return bytearray(struct.pack("!H", length))


def readInt16(buf):
    # deserialize a 16 bit integer from network format
    return struct.unpack("!H", buf[:2])[0]


def writeInt32(length):
    # serialize a 32 bit integer to network format
    return bytearray(struct.pack("!L", length))


def readInt32(buf):
    # deserialize a 32 bit integer from network format
    return struct.unpack("!L", buf[:4])[0]


def writeUTF(data):
    # data could be a string, or bytes.  If string, encode into bytes with utf-8
    if sys.version_info[0] < 3:
        data = bytearray(data, 'utf-8')
    else:
        data = data if type(data) == type(b"") else bytes(data, "utf-8")
    return writeInt16(len(data)) + data


def readUTF(buffer, maxlen):
    if maxlen >= 2:
        length = readInt16(buffer)
    else:
        raise MalformedPacket("Not enough data to read string length")
    maxlen -= 2
    if length > maxlen:
        raise MalformedPacket("Length delimited string too long")
    buf = buffer[2:2+length].decode("utf-8")
    # look for chars which are invalid for MQTT
    for c in buf: # look for D800-DFFF in the UTF string
        ord_c = ord(c)
        if ord_c >= 0xD800 and ord_c <= 0xDFFF:
            raise MalformedPacket("[MQTT-1.5.4-1] D800-DFFF found in UTF-8 data")
        if ord_c == 0x00: # look for null in the UTF string
            raise MalformedPacket("[MQTT-1.5.4-2] Null found in UTF-8 data")
        if ord_c == 0xFEFF:
            raise MalformedPacket("[MQTT-1.5.4-3] U+FEFF in UTF-8 data")
    return buf, length+2


def writeBytes(buffer):
    return writeInt16(len(buffer)) + buffer


def readBytes(buffer):
    length = readInt16(buffer)
    return buffer[2:2+length], length+2


class VariableByteIntegers:  # Variable Byte Integer
    """
    MQTT variable byte integer helper class.  Used
    in several places in MQTT v5.0 properties.

    """

    @staticmethod
    def encode(x):
        """
          Convert an integer 0 <= x <= 268435455 into multi-byte format.
          Returns the buffer convered from the integer.
        """
        assert 0 <= x <= 268435455
        buffer = b''
        while 1:
            digit = x % 128
            x //= 128
            if x > 0:
                digit |= 0x80
            if sys.version_info[0] >= 3:
                buffer += bytes([digit])
            else:
                buffer += bytes(chr(digit))
            if x == 0:
                break
        return buffer

    @staticmethod
    def decode(buffer):
        """
          Get the value of a multi-byte integer from a buffer
          Return the value, and the number of bytes used.

          [MQTT-1.5.5-1] the encoded value MUST use the minimum number of bytes necessary to represent the value
        """
        multiplier = 1
        value = 0
        bytes = 0
        while 1:
            bytes += 1
            digit = buffer[0]
            buffer = buffer[1:]
            value += (digit & 127) * multiplier
            if digit & 128 == 0:
                break
            multiplier *= 128
        return (value, bytes)


class Properties(object):
    """MQTT v5.0 properties class.

    See Properties.names for a list of accepted property names along with their numeric values.

    See Properties.properties for the data type of each property.

    Example of use:

        publish_properties = Properties(PacketTypes.PUBLISH)
        publish_properties.UserProperty = ("a", "2")
        publish_properties.UserProperty = ("c", "3")

    First the object is created with packet type as argument, no properties will be present at
    this point.  Then properties are added as attributes, the name of which is the string property
    name without the spaces.

    """

    def __init__(self, packetType):
        self.packetType = packetType
        self.types = ["Byte", "Two Byte Integer", "Four Byte Integer", "Variable Byte Integer",
                      "Binary Data", "UTF-8 Encoded String", "UTF-8 String Pair"]

        self.names = {
            "Payload Format Indicator": 1,
            "Message Expiry Interval": 2,
            "Content Type": 3,
            "Response Topic": 8,
            "Correlation Data": 9,
            "Subscription Identifier": 11,
            "Session Expiry Interval": 17,
            "Assigned Client Identifier": 18,
            "Server Keep Alive": 19,
            "Authentication Method": 21,
            "Authentication Data": 22,
            "Request Problem Information": 23,
            "Will Delay Interval": 24,
            "Request Response Information": 25,
            "Response Information": 26,
            "Server Reference": 28,
            "Reason String": 31,
            "Receive Maximum": 33,
            "Topic Alias Maximum": 34,
            "Topic Alias": 35,
            "Maximum QoS": 36,
            "Retain Available": 37,
            "User Property": 38,
            "Maximum Packet Size": 39,
            "Wildcard Subscription Available": 40,
            "Subscription Identifier Available": 41,
            "Shared Subscription Available": 42
        }

        self.properties = {
            # id:  type, packets
            # payload format indicator
            1: (self.types.index("Byte"), [PacketTypes.PUBLISH, PacketTypes.WILLMESSAGE]),
            2: (self.types.index("Four Byte Integer"), [PacketTypes.PUBLISH, PacketTypes.WILLMESSAGE]),
            3: (self.types.index("UTF-8 Encoded String"), [PacketTypes.PUBLISH, PacketTypes.WILLMESSAGE]),
            8: (self.types.index("UTF-8 Encoded String"), [PacketTypes.PUBLISH, PacketTypes.WILLMESSAGE]),
            9: (self.types.index("Binary Data"), [PacketTypes.PUBLISH, PacketTypes.WILLMESSAGE]),
            11: (self.types.index("Variable Byte Integer"),
                 [PacketTypes.PUBLISH, PacketTypes.SUBSCRIBE]),
            17: (self.types.index("Four Byte Integer"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK, PacketTypes.DISCONNECT]),
            18: (self.types.index("UTF-8 Encoded String"), [PacketTypes.CONNACK]),
            19: (self.types.index("Two Byte Integer"), [PacketTypes.CONNACK]),
            21: (self.types.index("UTF-8 Encoded String"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK, PacketTypes.AUTH]),
            22: (self.types.index("Binary Data"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK, PacketTypes.AUTH]),
            23: (self.types.index("Byte"),
                 [PacketTypes.CONNECT]),
            24: (self.types.index("Four Byte Integer"), [PacketTypes.WILLMESSAGE]),
            25: (self.types.index("Byte"), [PacketTypes.CONNECT]),
            26: (self.types.index("UTF-8 Encoded String"), [PacketTypes.CONNACK]),
            28: (self.types.index("UTF-8 Encoded String"),
                 [PacketTypes.CONNACK, PacketTypes.DISCONNECT]),
            31: (self.types.index("UTF-8 Encoded String"),
                 [PacketTypes.CONNACK, PacketTypes.PUBACK, PacketTypes.PUBREC,
                  PacketTypes.PUBREL, PacketTypes.PUBCOMP, PacketTypes.SUBACK,
                  PacketTypes.UNSUBACK, PacketTypes.DISCONNECT, PacketTypes.AUTH]),
            33: (self.types.index("Two Byte Integer"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK]),
            34: (self.types.index("Two Byte Integer"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK]),
            35: (self.types.index("Two Byte Integer"), [PacketTypes.PUBLISH]),
            36: (self.types.index("Byte"), [PacketTypes.CONNACK]),
            37: (self.types.index("Byte"), [PacketTypes.CONNACK]),
            38: (self.types.index("UTF-8 String Pair"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK,
                  PacketTypes.PUBLISH, PacketTypes.PUBACK,
                  PacketTypes.PUBREC, PacketTypes.PUBREL, PacketTypes.PUBCOMP,
                  PacketTypes.SUBSCRIBE, PacketTypes.SUBACK,
                  PacketTypes.UNSUBSCRIBE, PacketTypes.UNSUBACK,
                  PacketTypes.DISCONNECT, PacketTypes.AUTH, PacketTypes.WILLMESSAGE]),
            39: (self.types.index("Four Byte Integer"),
                 [PacketTypes.CONNECT, PacketTypes.CONNACK]),
            40: (self.types.index("Byte"), [PacketTypes.CONNACK]),
            41: (self.types.index("Byte"), [PacketTypes.CONNACK]),
            42: (self.types.index("Byte"), [PacketTypes.CONNACK]),
        }

    def allowsMultiple(self, compressedName):
        return self.getIdentFromName(compressedName) in [11, 38]

    def getIdentFromName(self, compressedName):
        # return the identifier corresponding to the property name
        result = -1
        for name in self.names.keys():
            if compressedName == name.replace(' ', ''):
                result = self.names[name]
                break
        return result

    def __setattr__(self, name, value):
        name = name.replace(' ', '')
        privateVars = ["packetType", "types", "names", "properties"]
        if name in privateVars:
            object.__setattr__(self, name, value)
        else:
            # the name could have spaces in, or not.  Remove spaces before assignment
            if name not in [aname.replace(' ', '') for aname in self.names.keys()]:
                raise MQTTException(
                    "Property name must be one of "+str(self.names.keys()))
            # check that this attribute applies to the packet type
            if self.packetType not in self.properties[self.getIdentFromName(name)][1]:
                raise MQTTException("Property %s does not apply to packet type %s"
                                    % (name, PacketTypes.Names[self.packetType]))
            if self.allowsMultiple(name):
                if type(value) != type([]):
                    value = [value]
                if hasattr(self, name):
                    value = object.__getattribute__(self, name) + value
            object.__setattr__(self, name, value)

    def __str__(self):
        buffer = "["
        first = True
        for name in self.names.keys():
            compressedName = name.replace(' ', '')
            if hasattr(self, compressedName):
                if not first:
                    buffer += ", "
                buffer += compressedName + " : " + \
                    str(getattr(self, compressedName))
                first = False
        buffer += "]"
        return buffer

    def json(self):
        data = {}
        for name in self.names.keys():
            compressedName = name.replace(' ', '')
            if hasattr(self, compressedName):
                data[compressedName] = getattr(self, compressedName)
        return data

    def isEmpty(self):
        rc = True
        for name in self.names.keys():
            compressedName = name.replace(' ', '')
            if hasattr(self, compressedName):
                rc = False
                break
        return rc

    def clear(self):
        for name in self.names.keys():
            compressedName = name.replace(' ', '')
            if hasattr(self, compressedName):
                delattr(self, compressedName)

    def writeProperty(self, identifier, type, value):
        buffer = b""
        buffer += VariableByteIntegers.encode(identifier)  # identifier
        if type == self.types.index("Byte"):  # value
            if sys.version_info[0] < 3:
                buffer += chr(value)
            else:
                buffer += bytes([value])
        elif type == self.types.index("Two Byte Integer"):
            buffer += writeInt16(value)
        elif type == self.types.index("Four Byte Integer"):
            buffer += writeInt32(value)
        elif type == self.types.index("Variable Byte Integer"):
            buffer += VariableByteIntegers.encode(value)
        elif type == self.types.index("Binary Data"):
            buffer += writeBytes(value)
        elif type == self.types.index("UTF-8 Encoded String"):
            buffer += writeUTF(value)
        elif type == self.types.index("UTF-8 String Pair"):
            buffer += writeUTF(value[0]) + writeUTF(value[1])
        return buffer

    def pack(self):
        # serialize properties into buffer for sending over network
        buffer = b""
        for name in self.names.keys():
            compressedName = name.replace(' ', '')
            if hasattr(self, compressedName):
                identifier = self.getIdentFromName(compressedName)
                attr_type = self.properties[identifier][0]
                if self.allowsMultiple(compressedName):
                    for prop in getattr(self, compressedName):
                        buffer += self.writeProperty(identifier,
                                                     attr_type, prop)
                else:
                    buffer += self.writeProperty(identifier, attr_type,
                                                 getattr(self, compressedName))
        return VariableByteIntegers.encode(len(buffer)) + buffer

    def readProperty(self, buffer, type, propslen):
        if type == self.types.index("Byte"):
            value = buffer[0]
            valuelen = 1
        elif type == self.types.index("Two Byte Integer"):
            value = readInt16(buffer)
            valuelen = 2
        elif type == self.types.index("Four Byte Integer"):
            value = readInt32(buffer)
            valuelen = 4
        elif type == self.types.index("Variable Byte Integer"):
            value, valuelen = VariableByteIntegers.decode(buffer)
        elif type == self.types.index("Binary Data"):
            value, valuelen = readBytes(buffer)
        elif type == self.types.index("UTF-8 Encoded String"):
            value, valuelen = readUTF(buffer, propslen)
        elif type == self.types.index("UTF-8 String Pair"):
            value, valuelen = readUTF(buffer, propslen)
            buffer = buffer[valuelen:]  # strip the bytes used by the value
            value1, valuelen1 = readUTF(buffer, propslen - valuelen)
            value = (value, value1)
            valuelen += valuelen1
        return value, valuelen

    def getNameFromIdent(self, identifier):
        rc = None
        for name in self.names:
            if self.names[name] == identifier:
                rc = name
        return rc

    def unpack(self, buffer):
        if sys.version_info[0] < 3:
            buffer = bytearray(buffer)
        self.clear()
        # deserialize properties into attributes from buffer received from network
        propslen, VBIlen = VariableByteIntegers.decode(buffer)
        buffer = buffer[VBIlen:]  # strip the bytes used by the VBI
        propslenleft = propslen
        while propslenleft > 0:  # properties length is 0 if there are none
            identifier, VBIlen = VariableByteIntegers.decode(
                buffer)  # property identifier
            buffer = buffer[VBIlen:]  # strip the bytes used by the VBI
            propslenleft -= VBIlen
            attr_type = self.properties[identifier][0]
            value, valuelen = self.readProperty(
                buffer, attr_type, propslenleft)
            buffer = buffer[valuelen:]  # strip the bytes used by the value
            propslenleft -= valuelen
            propname = self.getNameFromIdent(identifier)
            compressedName = propname.replace(' ', '')
            if not self.allowsMultiple(compressedName) and hasattr(self, compressedName):
                raise MQTTException(
                    "Property '%s' must not exist more than once" % property)
            setattr(self, propname, value)
        return self, propslen + VBIlen



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2016 James Myatt <james@jamesmyatt.co.uk>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    James Myatt - initial implementation

# This shows a simple example of standard logging with an MQTT subscriber client.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.client as mqtt

import logging
logging.basicConfig(level=logging.DEBUG)

# If you want to use a specific client id, use
# mqttc = mqtt.Client("client-id")
# but note that the client id must be unique on the broker. Leaving the client
# id parameter empty will generate a random id for you.
mqttc = mqtt.Client()

logger = logging.getLogger(__name__)
mqttc.enable_logger(logger)

mqttc.connect("mqtt.eclipse.org", 1883, 60)
mqttc.subscribe("$SYS/#", 0)

mqttc.loop_forever()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Roger Light <roger@atchoo.org>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Distribution License v1.0
# which accompanies this distribution.
#
# The Eclipse Distribution License is available at
#   http://www.eclipse.org/org/documents/edl-v10.php.
#
# Contributors:
#    Roger Light - initial implementation
# Copyright (c) 2014 Roger Light <roger@atchoo.org>
# All rights reserved.

# This demonstrates the session present flag when connecting.

import context  # Ensures paho is in PYTHONPATH
import paho.mqtt.client as mqtt


def on_connect(mqttc, obj, flags, rc):
    if obj == 0:
        print("First connection:")
    elif obj == 1:
        print("Second connection:")
    elif obj == 2:
        print("Third connection (with clean session=True):")
    print("    Session present: " + str(flags['session present']))
    print("    Connection result: " + str(rc))
    mqttc.disconnect()


def on_disconnect(mqttc, obj, rc):
    mqttc.user_data_set(obj + 1)
    if obj == 0:
        mqttc.reconnect()


def on_log(mqttc, obj, level, string):
    print(string)


mqttc = mqtt.Client(client_id="asdfj", clean_session=False)
mqttc.on_connect = on_connect
mqttc.on_disconnect = on_disconnect
# Uncomment to enable debug messages
# mqttc.on_log = on_log
mqttc.user_data_set(0)
mqttc.connect("mqtt.eclipse.org", 1883, 60)

mqttc.loop_forever()

# Clear session
mqttc = mqtt.Client(client_id="asdfj", clean_session=True)
mqttc.on_connect = on_connect
mqttc.user_data_set(2)
mqttc.connect("mqtt.eclipse.org", 1883, 60)
mqttc.loop_forever()



# -*- coding: utf-8 -*-

# Ensure can import paho package
try:
    import paho

except ImportError:
    # This part is only required to run the examples from within the examples
    # directory when the module itself is not installed.
    import sys
    import os
    import inspect

    cmd_subfolder = os.path.realpath(
        os.path.abspath(
            os.path.join(
                os.path.split(
                    inspect.getfile(inspect.currentframe())
                )[0],
                "..",
                "src"
            )
        )
    )
    if cmd_subfolder not in sys.path:
        sys.path.insert(0, cmd_subfolder)

    import paho



#!/usr/bin/env python

# Test whether a client produces a correct connect and subsequent disconnect when using SSL.
# Client must provide a certificate.

# The client should connect to port 1888 with keepalive=60, clean session set,
# and client id 08-ssl-connect-crt-auth
# It should use the CA certificate ssl/all-ca.crt for verifying the server.
# The test will send a CONNACK message to the client with rc=0. Upon receiving
# the CONNACK and verifying that rc=0, the client should send a DISCONNECT
# message. If rc!=0, the client should exit with an error.

import context
import paho_test
from paho_test import ssl

context.check_ssl()

rc = 1
keepalive = 60
connect_packet = paho_test.gen_connect("08-ssl-connect-crt-auth", keepalive=keepalive)
connack_packet = paho_test.gen_connack(rc=0)
disconnect_packet = paho_test.gen_disconnect()

ssock = paho_test.create_server_socket_ssl(cert_reqs=ssl.CERT_REQUIRED)

client = context.start_client()

try:
    (conn, address) = ssock.accept()
    conn.settimeout(10)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        conn.send(connack_packet)

        if paho_test.expect_packet(conn, "disconnect", disconnect_packet):
            rc = 0

    conn.close()
finally:
    client.terminate()
    client.wait()
    ssock.close()

exit(rc)



import socket
from six.moves import socketserver
import threading
import contextlib

import pytest


class FakeBroker:
    def __init__(self):
        # Bind to "localhost" for maximum performance, as described in:
        # http://docs.python.org/howto/sockets.html#ipc
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.settimeout(30)
        sock.bind(("localhost", 1888))
        sock.listen(1)

        self._sock = sock
        self._conn = None

    def start(self):
        if self._sock is None:
            raise ValueError('Socket is not open')

        (conn, address) = self._sock.accept()
        conn.settimeout(10)
        self._conn = conn

    def finish(self):
        if self._conn is not None:
            self._conn.close()
            self._conn = None

        if self._sock is not None:
            self._sock.close()
            self._sock = None

    def receive_packet(self, num_bytes):
        if self._conn is None:
            raise ValueError('Connection is not open')

        packet_in = self._conn.recv(num_bytes)
        return packet_in

    def send_packet(self, packet_out):
        if self._conn is None:
            raise ValueError('Connection is not open')

        count = self._conn.send(packet_out)
        return count


@pytest.fixture
def fake_broker():
    # print('Setup broker')
    broker = FakeBroker()

    yield broker

    # print('Teardown broker')
    broker.finish()


class ThreadedTCPServer(socketserver.ThreadingMixIn, socketserver.TCPServer):
    pass


class FakeWebsocketBroker(threading.Thread):
    def __init__(self):
        super(FakeWebsocketBroker, self).__init__()

        self.host = "localhost"
        self.port = 1888

        self._server = None
        self._running = True
        self.handler_cls = False

    @contextlib.contextmanager
    def serve(self, tcphandler):
        self._server = ThreadedTCPServer((self.host, self.port), tcphandler)

        try:
            self.start()

            if not self._running:
                raise RuntimeError("Error starting server")
            yield
        finally:
            if self._server:
                self._server.shutdown()
                self._server.server_close()

    def run(self):
        self._running = True
        self._server.serve_forever()


@pytest.fixture
def fake_websocket_broker():
    socketserver.TCPServer.allow_reuse_address = True

    broker = FakeWebsocketBroker()

    yield broker



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from setuptools import setup, find_packages

sys.path.insert(0, 'src')
from paho.mqtt import __version__

with open('README.rst', 'rb') as readme_file:
    readme = readme_file.read().decode('utf-8')

requirements = []
test_requirements = ['pytest', 'pylama', 'six']
needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
setup_requirements = ['pytest-runner'] if needs_pytest else []
extra_requirements = {'proxy': ['PySocks']}

if sys.version_info < (3, 0):
    test_requirements += ['mock']

setup(
    name='paho-mqtt',
    version=__version__,
    description='MQTT version 3.1.1 client class',
    long_description=readme,
    author='Roger Light',
    author_email='roger@atchoo.org',
    url='http://eclipse.org/paho',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    install_requires=requirements,
    license='Eclipse Public License v1.0 / Eclipse Distribution License v1.0',
    zip_safe=False,
    keywords='paho',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Communications',
        'Topic :: Internet',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
    extras_require=extra_requirements
)



#!/usr/bin/env python

# Test whether a client sends a correct UNSUBSCRIBE packet.

import context
import paho_test

rc = 1
keepalive = 60
connect_packet = paho_test.gen_connect("unsubscribe-test", keepalive=keepalive)
connack_packet = paho_test.gen_connack(rc=0)

disconnect_packet = paho_test.gen_disconnect()

mid = 1
unsubscribe_packet = paho_test.gen_unsubscribe(mid, "unsubscribe/test")
unsuback_packet = paho_test.gen_unsuback(mid)

sock = paho_test.create_server_socket()

client = context.start_client()

try:
    (conn, address) = sock.accept()
    conn.settimeout(10)

    if paho_test.expect_packet(conn, "connect", connect_packet):
        conn.send(connack_packet)

        if paho_test.expect_packet(conn, "unsubscribe", unsubscribe_packet):
            conn.send(unsuback_packet)

            if paho_test.expect_packet(conn, "disconnect", disconnect_packet):
                rc = 0

    conn.close()
finally:
    client.terminate()
    client.wait()
    sock.close()

exit(rc)
